/** \file
 * Библиотека криптографических алгоритмов ГОСТ.
 */

#pragma once

#define ALIGNED(n) __attribute__((aligned(n)))

#ifndef GOSTCRYPTO_H
#define GOSTCRYPTO_H

#include <stdint.h>
#include <stdbool.h>

/**
 * Контекст блочного шифра для расшифрования с длиной блока 128 бит.
 * Определён в ГОСТ Р 34.12-2018 (Кузнечик).
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_dec_new
 * - #kuznyechik_dec_from_enc
 * - #kuznyechik_dec_decrypt_block
 * - #kuznyechik_dec_decrypt_blocks
 * - #kuznyechik_dec_ecb_decrypt_unpad
 * - #kuznyechik_dec_drop
 *
 * Набор типов для работы с шифром:
 *
 * - #KuznyechikKey
 * - #KuznyechikBlock
 */
typedef struct ALIGNED(16) KuznyechikDec {
  uint8_t p[160];
} KuznyechikDec;

/**
 * Ключ шифров #KuznyechikEnc и #KuznyechikDec.
 */
typedef uint8_t KuznyechikKey[32];

/**
 * Контекст блочного шифра для зашифрования с длиной блока 128 бит.
 * Определён в ГОСТ Р 34.12-2018 (Кузнечик).
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_enc_new
 * - #kuznyechik_enc_encrypt_block
 * - #kuznyechik_enc_encrypt_blocks
 * - #kuznyechik_enc_ecb_encrypt_pad
 * - #kuznyechik_enc_drop
 *
 * Набор типов для работы с шифром:
 *
 * - #KuznyechikKey
 * - #KuznyechikBlock
 */
typedef struct ALIGNED(16) KuznyechikEnc {
  uint8_t p[160];
} KuznyechikEnc;

/**
 * Блок шифров #KuznyechikEnc и #KuznyechikDec.
 */
typedef uint8_t KuznyechikBlock[16];

/**
 * Блочный шифр с длиной блока 64 бит. Определён в ГОСТ Р 34.12-2015 (Магма), ГОСТ 34.12-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_new
 * - #magma_encrypt_block
 * - #magma_decrypt_block
 * - #magma_encrypt_blocks
 * - #magma_decrypt_blocks
 * - #magma_drop
 *
 * Набор типов для работы с шифром:
 *
 * - #MagmaKey
 * - #MagmaBlock
 */
typedef struct ALIGNED(4) Magma {
  uint8_t p[32];
} Magma;

/**
 * Ключ шифра #Magma.
 */
typedef uint8_t MagmaKey[32];

/**
 * Блок шифра #Magma
 */
typedef uint8_t MagmaBlock[8];

/**
 * Ключ формирования цифровой подписи согласно ГОСТ 34.10-2018
 * с использованием эллиптической кривой `id-tc26-gost-3410-12-256-paramSetA`
 * заданной в Р 1323565.1.024-2019.
 */
typedef struct ALIGNED(8) Ec2012_256A_SignatureKey {
  uint8_t p[32];
} Ec2012_256A_SignatureKey;

/**
 * Ключ проверки цифровой подписи согласно ГОСТ 34.10-2018
 * с использованием эллиптической кривой `id-tc26-gost-3410-12-256-paramSetA`
 * заданной в Р 1323565.1.024-2019.
 */
typedef struct ALIGNED(8) Ec2012_256A_VerificationKey {
  uint8_t p[96];
} Ec2012_256A_VerificationKey;

/**
 * Байты представляющие сериализованный скаляр 256-битной эллиптической кривой.
 */
typedef uint8_t Ec256ScalarBytes[32];

/**
 * Байты представляющие сериализованный двойной скаляр 256-битной эллиптической кривой.
 */
typedef uint8_t Ec256DoubleScalarBytes[64];

/**
 * Хеш-код возвращаемый #Streebog256
 */
typedef uint8_t Streebog256Result[32];

/**
 * Ключ формирования цифровой подписи согласно ГОСТ 34.10-2018
 * с использованием эллиптической кривой `id-tc26-gost-3410-12-256-paramSetB`
 * заданной в Р 1323565.1.024-2019.
 */
typedef struct ALIGNED(8) Ec2012_256B_SignatureKey {
  uint8_t p[32];
} Ec2012_256B_SignatureKey;

/**
 * Ключ проверки цифровой подписи согласно ГОСТ 34.10-2018
 * с использованием эллиптической кривой `id-tc26-gost-3410-12-256-paramSetB`
 * заданной в Р 1323565.1.024-2019.
 */
typedef struct ALIGNED(8) Ec2012_256B_VerificationKey {
  uint8_t p[96];
} Ec2012_256B_VerificationKey;

/**
 * Ключ формирования цифровой подписи согласно ГОСТ 34.10-2018
 * с использованием эллиптической кривой `id-tc26-gost-3410-12-256-paramSetC`
 * заданной в Р 1323565.1.024-2019.
 */
typedef struct ALIGNED(8) Ec2012_256C_SignatureKey {
  uint8_t p[32];
} Ec2012_256C_SignatureKey;

/**
 * Ключ проверки цифровой подписи согласно ГОСТ 34.10-2018
 * с использованием эллиптической кривой `id-tc26-gost-3410-12-256-paramSetC`
 * заданной в Р 1323565.1.024-2019.
 */
typedef struct ALIGNED(8) Ec2012_256C_VerificationKey {
  uint8_t p[96];
} Ec2012_256C_VerificationKey;

/**
 * Ключ формирования цифровой подписи согласно ГОСТ 34.10-2018
 * с использованием эллиптической кривой `id-tc26-gost-3410-12-256-paramSetD`
 * заданной в Р 1323565.1.024-2019.
 */
typedef struct ALIGNED(8) Ec2012_256D_SignatureKey {
  uint8_t p[32];
} Ec2012_256D_SignatureKey;

/**
 * Ключ проверки цифровой подписи согласно ГОСТ 34.10-2018
 * с использованием эллиптической кривой `id-tc26-gost-3410-12-256-paramSetD`
 * заданной в Р 1323565.1.024-2019.
 */
typedef struct ALIGNED(8) Ec2012_256D_VerificationKey {
  uint8_t p[96];
} Ec2012_256D_VerificationKey;

/**
 * Ключ формирования цифровой подписи согласно ГОСТ 34.10-2018
 * с использованием эллиптической кривой `id-tc26-gost-3410-12-512-paramSetA`
 * заданной в Р 1323565.1.024-2019.
 */
typedef struct ALIGNED(8) Ec2012_512A_SignatureKey {
  uint8_t p[64];
} Ec2012_512A_SignatureKey;

/**
 * Ключ проверки подписи для формирования цифровой подписи согласно ГОСТ 34.10-2012 и ГОСТ 34.10-2018
 * с использованием эллиптической кривой `id-tc26-gost-3410-12-512-paramSetA` заданной в Р 1323565.1.024-2019.
 */
typedef struct ALIGNED(8) Ec2012_512A_VerificationKey {
  uint8_t p[192];
} Ec2012_512A_VerificationKey;

/**
 * Байты представляющие сериализованный скаляр 512-битной эллиптической кривой.
 */
typedef uint8_t Ec512ScalarBytes[64];

/**
 * Байты представляющие сериализованный двойной скаляр 512-битной эллиптической кривой.
 */
typedef uint8_t Ec512DoubleScalarBytes[128];

/**
 * Хеш-код возвращаемый #Streebog512
 */
typedef uint8_t Streebog512Result[64];

/**
 * Ключ формирования цифровой подписи согласно ГОСТ 34.10-2018
 * с использованием эллиптической кривой `id-tc26-gost-3410-12-512-paramSetB`
 * заданной в Р 1323565.1.024-2019.
 */
typedef struct ALIGNED(8) Ec2012_512B_SignatureKey {
  uint8_t p[64];
} Ec2012_512B_SignatureKey;

/**
 * Ключ проверки подписи для формирования цифровой подписи согласно ГОСТ 34.10-2012 и ГОСТ 34.10-2018
 * с использованием эллиптической кривой `id-tc26-gost-3410-12-512-paramSetB` заданной в Р 1323565.1.024-2019.
 */
typedef struct ALIGNED(8) Ec2012_512B_VerificationKey {
  uint8_t p[192];
} Ec2012_512B_VerificationKey;

/**
 * Ключ формирования цифровой подписи согласно ГОСТ 34.10-2018
 * с использованием эллиптической кривой `id-tc26-gost-3410-12-512-paramSetC`
 * заданной в Р 1323565.1.024-2019.
 */
typedef struct ALIGNED(8) Ec2012_512C_SignatureKey {
  uint8_t p[64];
} Ec2012_512C_SignatureKey;

/**
 * Ключ проверки подписи для формирования цифровой подписи согласно ГОСТ 34.10-2012 и ГОСТ 34.10-2018
 * с использованием эллиптической кривой `id-tc26-gost-3410-12-512-paramSetC` заданной в Р 1323565.1.024-2019.
 */
typedef struct ALIGNED(8) Ec2012_512C_VerificationKey {
  uint8_t p[192];
} Ec2012_512C_VerificationKey;

/**
 * Функция хэширования с длиной хэш-кода 256 бит. Определена в ГОСТ Р 34.11-2012 (Стрибог) и ГОСТ 34.11-2018.
 *
 * Набор функций для работы с функцией хэширования:
 *
 * - #streebog256_new
 * - #streebog256_digest
 * - #streebog256_update
 * - #streebog256_finalize
 * - #streebog256_finalize_reset
 * - #streebog256_reset
 * - #streebog256_drop
 *
 * Набор типов для работы с функцией хэширования:
 *
 * - #Streebog256Result
 */
typedef struct ALIGNED(8) Streebog256 {
  uint8_t p[264];
} Streebog256;

/**
 * Функция хэширования с длиной хэш-кода 512 бит. Определена в ГОСТ Р 34.11-2012 (Стрибог) и ГОСТ 34.11-2018.
 *
 * Набор функций для работы с функцией хэширования:
 *
 * - #streebog512_new
 * - #streebog512_digest
 * - #streebog512_update
 * - #streebog512_finalize
 * - #streebog512_finalize_reset
 * - #streebog512_reset
 * - #streebog512_drop
 *
 * Набор типов для работы с функцией хэширования:
 *
 * - #Streebog512Result
 */
typedef struct ALIGNED(8) Streebog512 {
  uint8_t p[264];
} Streebog512;

/**
 * Имитовставка для режимов использующих шифр #Streebog256Hmac.
 */
typedef uint8_t Streebog256HmacTag[32];

/**
 * Режим выработки имитовставки с использованием шифра с длиной блока 128 бит. Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с режимом выработки имитовставки:
 *
 * - #kuznyechik_cmac_new
 * - #kuznyechik_cmac_update
 * - #kuznyechik_cmac_finalize
 * - #kuznyechik_cmac_finalize_reset
 * - #kuznyechik_cmac_verify
 * - #kuznyechik_cmac_reset
 * - #kuznyechik_cmac_drop
 *
 * Набор типов для работы с режимом выработки имитовставки:
 *
 * - #KuznyechikKey
 * - #KuznyechikTag
 */
typedef struct ALIGNED(16) KuznyechikCmac {
  uint8_t p[208];
} KuznyechikCmac;

/**
 * Имитовставка для режимов использующих шифры #KuznyechikEnc и #KuznyechikDec.
 */
typedef KuznyechikBlock KuznyechikTag;

/**
 * Режим выработки имитовставки с использованием шифра с длиной блока 128 бит. Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с режимом выработки имитовставки:
 *
 * - #kuznyechik_ref_cmac_new
 * - #kuznyechik_ref_cmac_update
 * - #kuznyechik_ref_cmac_finalize
 * - #kuznyechik_ref_cmac_finalize_reset
 * - #kuznyechik_ref_cmac_verify
 * - #kuznyechik_ref_cmac_reset
 * - #kuznyechik_ref_cmac_drop
 *
 * Набор типов для работы с режимом выработки имитовставки:
 *
 * - #Kuznyechik
 * - #KuznyechikTag
 */
typedef struct ALIGNED(8) KuznyechikRefCmac {
  uint8_t p[48];
} KuznyechikRefCmac;

/**
 * Режим выработки имитовставки с использованием шифра с длиной блока 64 бит
 * владеющий контекстом блочного шифра. Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с режимом выработки имитовставки:
 *
 * - #magma_cmac_new
 * - #magma_cmac_update
 * - #magma_cmac_finalize
 * - #magma_cmac_finalize_reset
 * - #magma_cmac_verify
 * - #magma_cmac_reset
 * - #magma_cmac_drop
 *
 * Набор типов для работы с режимом выработки имитовставки:
 *
 * - #MagmaKey
 * - #MagmaTag
 */
typedef struct ALIGNED(4) MagmaCmac {
  uint8_t p[52];
} MagmaCmac;

/**
 * Имитовставка для режимов использующих шифр #Magma.
 */
typedef MagmaBlock MagmaTag;

/**
 * Режим выработки имитовставки с использованием шифра с длиной блока 64 бит
 * использующий ссылку на блочный шифр. Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с режимом выработки имитовставки:
 *
 * - #magma_ref_cmac_new
 * - #magma_ref_cmac_update
 * - #magma_ref_cmac_finalize
 * - #magma_ref_cmac_finalize_reset
 * - #magma_ref_cmac_verify
 * - #magma_ref_cmac_reset
 * - #magma_ref_cmac_drop
 *
 * Набор типов для работы с режимом выработки имитовставки:
 *
 * - #Magma
 * - #MagmaTag
 */
typedef struct ALIGNED(8) MagmaRefCmac {
  uint8_t p[32];
} MagmaRefCmac;

/**
 * Режим выработки имитовставки HMAC с использованием хеш-функции Стрибог-256.
 *
 * Определен в Р 50.1.113-2016.
 *
 * Набор функций для работы с режимом выработки имитовставки:
 *
 * - #streebog256_hmac_new
 * - #streebog256_hmac_update
 * - #streebog256_hmac_finalize
 * - #streebog256_hmac_finalize_reset
 * - #streebog256_hmac_verify
 * - #streebog256_hmac_reset
 * - #streebog256_hmac_drop
 *
 * Набор типов для работы с режимом выработки имитовставки:
 *
 * - #Streebog256HmacTag
 */
typedef struct ALIGNED(8) Streebog256Hmac {
  uint8_t p[456];
} Streebog256Hmac;

/**
 * Режим выработки имитовставки HMAC с использованием хеш-функции Стрибог-512.
 *
 * Определен в Р 50.1.113-2016.
 *
 * Набор функций для работы с режимом выработки имитовставки:
 *
 * - #streebog512_hmac_new
 * - #streebog512_hmac_update
 * - #streebog512_hmac_finalize
 * - #streebog512_hmac_finalize_reset
 * - #streebog512_hmac_verify
 * - #streebog512_hmac_reset
 * - #streebog512_hmac_drop
 *
 * Набор типов для работы с режимом выработки имитовставки:
 *
 * - #Streebog512HmacTag
 */
typedef struct ALIGNED(8) Streebog512Hmac {
  uint8_t p[456];
} Streebog512Hmac;

/**
 * Имитовставка для режимов использующих шифр #Streebog512Hmac.
 */
typedef uint8_t Streebog512HmacTag[64];

/**
 * Синхропосылка MGM работающего с шифром #Kuznyechik.
 */
typedef KuznyechikBlock Mgm128Nonce;

/**
 * Имитовставка MGM работающего с шифром #Kuznyechik.
 */
typedef KuznyechikBlock Mgm128Tag;

/**
 * Синхропосылка MGM работающего с шифром #Magma.
 */
typedef MagmaBlock Mgm64Nonce;

/**
 * Имитовставка MGM работающего с шифром #Magma.
 */
typedef MagmaBlock Mgm64Tag;

/**
 * Блочный режим с длиной блока 128 бит в режиме простой замены с зацеплением
 * с размером синхропосылки 16 байт владеющий контекстом блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_cbc_dec_new
 * - #kuznyechik_cbc_decrypt_blocks
 * - #kuznyechik_cbc_decrypt_pad
 * - #kuznyechik_cbc_dec_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #KuznyechikKey
 * - #KuznyechikNonce
 * - #KuznyechikBlock
 */
typedef struct ALIGNED(16) KuznyechikCbcDec {
  uint8_t p[176];
} KuznyechikCbcDec;

/**
 * Синхропосылка для режимов использующих шифры #KuznyechikEnc и #KuznyechikDec
 * (за исключением CTR).
 */
typedef KuznyechikBlock KuznyechikNonce;

/**
 * Блочный режим с длиной блока 128 бит в режиме простой замены с зацеплением
 * с размером синхропосылки 16 байт владеющий контекстом блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_cbc_enc_new
 * - #kuznyechik_cbc_encrypt_blocks
 * - #kuznyechik_cbc_encrypt_pad
 * - #kuznyechik_cbc_enc_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #KuznyechikKey
 * - #KuznyechikNonce
 * - #KuznyechikBlock
 */
typedef struct ALIGNED(16) KuznyechikCbcEnc {
  uint8_t p[176];
} KuznyechikCbcEnc;

/**
 * Блочный режим с длиной блока 64 бит в режиме простой замены с зацеплением
 * с размером синхропосылки 8 байт владеющий контекстом блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_cbc_dec_new
 * - #magma_cbc_decrypt_blocks
 * - #magma_cbc_decrypt_pad
 * - #magma_cbc_dec_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #MagmaKey
 * - #MagmaNonce
 * - #MagmaBlock
 */
typedef struct ALIGNED(4) MagmaCbcDec {
  uint8_t p[40];
} MagmaCbcDec;

/**
 * Синхропосылка для режимов использующих шифр #Magma (за исключением CTR).
 */
typedef MagmaBlock MagmaNonce;

/**
 * Блочный режим с длиной блока 64 бит в режиме простой замены с зацеплением
 * с размером синхропосылки 8 байт владеющий контекстом блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_cbc_enc_new
 * - #magma_cbc_encrypt_blocks
 * - #magma_cbc_encrypt_pad
 * - #magma_cbc_enc_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #MagmaKey
 * - #MagmaNonce
 * - #MagmaBlock
 */
typedef struct ALIGNED(4) MagmaCbcEnc {
  uint8_t p[40];
} MagmaCbcEnc;

/**
 * Блочный режим с длиной блока 128 бит в режиме простой замены с зацеплением
 * с размером синхропосылки 16 байт использующий ссылку на контекст блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_cbc_dec_new
 * - #kuznyechik_cbc_decrypt_blocks
 * - #kuznyechik_cbc_decrypt_pad
 * - #kuznyechik_cbc_dec_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #KuznyechikKey
 * - #KuznyechikNonce
 * - #KuznyechikBlock
 */
typedef struct ALIGNED(8) KuznyechikRefCbcDec {
  uint8_t p[24];
} KuznyechikRefCbcDec;

/**
 * Блочный режим с длиной блока 128 бит в режиме простой замены с зацеплением
 * с размером синхропосылки 16 байт использующий ссылку на контекст блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_ref_cbc_enc_new
 * - #kuznyechik_ref_cbc_encrypt_blocks
 * - #kuznyechik_ref_cbc_encrypt_pad
 * - #kuznyechik_ref_cbc_enc_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #KuznyechikKey
 * - #KuznyechikNonce
 * - #KuznyechikBlock
 */
typedef struct ALIGNED(8) KuznyechikRefCbcEnc {
  uint8_t p[24];
} KuznyechikRefCbcEnc;

/**
 * Блочный режим с длиной блока 64 бит в режиме простой замены с зацеплением
 * с размером синхропосылки 8 байт использующий ссылку на контекст блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_cbc_dec_new
 * - #magma_cbc_decrypt_blocks
 * - #magma_cbc_decrypt_pad
 * - #magma_cbc_dec_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #MagmaKey
 * - #MagmaNonce
 * - #MagmaBlock
 */
typedef struct ALIGNED(8) MagmaRefCbcDec {
  uint8_t p[16];
} MagmaRefCbcDec;

/**
 * Блочный режим с длиной блока 64 бит в режиме простой замены с зацеплением
 * с размером синхропосылки 8 байт использующий ссылку на контекст блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_ref_cbc_enc_new
 * - #magma_ref_cbc_encrypt_blocks
 * - #magma_ref_cbc_encrypt_pad
 * - #magma_ref_cbc_enc_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #MagmaKey
 * - #MagmaNonce
 * - #MagmaBlock
 */
typedef struct ALIGNED(8) MagmaRefCbcEnc {
  uint8_t p[16];
} MagmaRefCbcEnc;

/**
 * Блочный режим с длиной блока 128 бит в режиме гаммирования с обратной связью по шифротексту
 * с размером синхропосылки 16 байт владеющий контекстом блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_cfb_dec_new
 * - #kuznyechik_cfb_decrypt_blocks
 * - #kuznyechik_cfb_decrypt_pad
 * - #kuznyechik_cfb_dec_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #KuznyechikKey
 * - #KuznyechikNonce
 * - #KuznyechikBlock
 */
typedef struct ALIGNED(16) KuznyechikCfbDec {
  uint8_t p[176];
} KuznyechikCfbDec;

/**
 * Блочный режим с длиной блока 128 бит в режиме гаммирования с обратной связью по шифротексту
 * с размером синхропосылки 16 байт владеющий контекстом блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_cfb_enc_new
 * - #kuznyechik_cfb_encrypt_blocks
 * - #kuznyechik_cfb_encrypt_pad
 * - #kuznyechik_cfb_enc_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #KuznyechikKey
 * - #KuznyechikNonce
 * - #KuznyechikBlock
 */
typedef struct ALIGNED(16) KuznyechikCfbEnc {
  uint8_t p[176];
} KuznyechikCfbEnc;

/**
 * Блочный режим с длиной блока 64 бит в режиме гаммирования с обратной связью по шифротексту
 * с размером синхропосылки 8 байт владеющий контекстом блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_cfb_dec_new
 * - #magma_cfb_decrypt_blocks
 * - #magma_cfb_decrypt_pad
 * - #magma_cfb_dec_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #MagmaKey
 * - #MagmaNonce
 * - #MagmaBlock
 */
typedef struct ALIGNED(4) MagmaCfbDec {
  uint8_t p[40];
} MagmaCfbDec;

/**
 * Блочный режим с длиной блока 64 бит в режиме гаммирования с обратной связью по шифротексту
 * с размером синхропосылки 8 байт владеющий контекстом блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_cfb_enc_new
 * - #magma_cfb_encrypt_blocks
 * - #magma_cfb_encrypt_pad
 * - #magma_cfb_enc_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #MagmaKey
 * - #MagmaNonce
 * - #MagmaBlock
 */
typedef struct ALIGNED(4) MagmaCfbEnc {
  uint8_t p[40];
} MagmaCfbEnc;

/**
 * Блочный режим с длиной блока 128 бит в режиме гаммирования с обратной связью по шифротексту
 * с размером синхропосылки 16 байт использующий ссылку на контекст блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_cfb_dec_new
 * - #kuznyechik_cfb_decrypt_blocks
 * - #kuznyechik_cfb_decrypt_pad
 * - #kuznyechik_cfb_dec_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #KuznyechikKey
 * - #KuznyechikNonce
 * - #KuznyechikBlock
 */
typedef struct ALIGNED(8) KuznyechikRefCfbDec {
  uint8_t p[24];
} KuznyechikRefCfbDec;

/**
 * Блочный режим с длиной блока 128 бит в режиме гаммирования с обратной связью по шифротексту
 * с размером синхропосылки 16 байт использующий ссылку на контекст блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_ref_cfb_enc_new
 * - #kuznyechik_ref_cfb_encrypt_blocks
 * - #kuznyechik_ref_cfb_encrypt_pad
 * - #kuznyechik_ref_cfb_enc_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #KuznyechikKey
 * - #KuznyechikNonce
 * - #KuznyechikBlock
 */
typedef struct ALIGNED(8) KuznyechikRefCfbEnc {
  uint8_t p[24];
} KuznyechikRefCfbEnc;

/**
 * Блочный режим с длиной блока 64 бит в режиме гаммирования с обратной связью по шифротексту
 * с размером синхропосылки 8 байт использующий ссылку на контекст блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_cfb_dec_new
 * - #magma_cfb_decrypt_blocks
 * - #magma_cfb_decrypt_pad
 * - #magma_cfb_dec_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #MagmaKey
 * - #MagmaNonce
 * - #MagmaBlock
 */
typedef struct ALIGNED(8) MagmaRefCfbDec {
  uint8_t p[16];
} MagmaRefCfbDec;

/**
 * Блочный режим с длиной блока 64 бит в режиме гаммирования с обратной связью по шифротексту
 * с размером синхропосылки 8 байт использующий ссылку на контекст блочного шифра.
 * Определен в ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_ref_cfb_enc_new
 * - #magma_ref_cfb_encrypt_blocks
 * - #magma_ref_cfb_encrypt_pad
 * - #magma_ref_cfb_enc_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #MagmaKey
 * - #MagmaNonce
 * - #MagmaBlock
 */
typedef struct ALIGNED(8) MagmaRefCfbEnc {
  uint8_t p[16];
} MagmaRefCfbEnc;

/**
 * Блочный режим с длиной блока 128 бит в режиме гаммирования владующий контекстом
 * блочного шифра. Реализует синхронный потоковый шифр. Определен в
 * ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_ctr_new
 * - #kuznyechik_ctr_apply_keystream
 * - #kuznyechik_ctr_seek
 * - #kuznyechik_ctr_get_pos
 * - #kuznyechik_ctr_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #KuznyechikKey
 * - #KuznyechikCtrNonce
 */
typedef struct ALIGNED(16) KuznyechikCtr {
  uint8_t p[224];
} KuznyechikCtr;

/**
 * Синхропосылка для режима CTR использующего шифры #KuznyechikEnc и #KuznyechikDec.
 */
typedef uint8_t KuznyechikCtrNonce[8];

/**
 * Блочный режим с длиной блока 64 бит в режиме гаммирования владующий контекстом
 * блочного шифра. Реализует синхронный потоковый шифр. Определен в
 * ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_ctr_new
 * - #magma_ctr_apply_keystream
 * - #magma_ctr_seek
 * - #magma_ctr_get_pos
 * - #magma_ctr_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #MagmaKey
 * - #MagmaCtrNonce
 */
typedef struct ALIGNED(8) MagmaCtr {
  uint8_t p[64];
} MagmaCtr;

/**
 * Синхропосылка для режима CTR использующего шифр #Magma.
 */
typedef uint8_t MagmaCtrNonce[4];

/**
 * Блочный режим с длиной блока 128 бит в режиме гаммирования использующий ссылку на
 * контекст блочного шифра. Реализует синхронный потоковый шифр. Определен в
 * ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_ref_ctr_new
 * - #kuznyechik_ref_ctr_apply_keystream
 * - #kuznyechik_ref_ctr_seek
 * - #kuznyechik_ref_ctr_get_pos
 * - #kuznyechik_ref_ctr_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #Kuznyechik
 * - #KuznyechikCtrNonce
 */
typedef struct ALIGNED(8) KuznyechikRefCtr {
  uint8_t p[56];
} KuznyechikRefCtr;

/**
 * Блочный режим с длиной блока 64 бит в режиме гаммирования использующий ссылку на
 * контекст блочного шифра. Реализует синхронный потоковый шифр. Определен в
 * ГОСТ 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_ref_ctr_new
 * - #magma_ref_ctr_apply_keystream
 * - #magma_ref_ctr_seek
 * - #magma_ref_ctr_get_pos
 * - #magma_ref_ctr_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #Magma
 * - #MagmaCtrNonce
 */
typedef struct ALIGNED(8) MagmaRefCtr {
  uint8_t p[40];
} MagmaRefCtr;

/**
 * Блочный режим с длиной блока 128 бит в режиме гаммирования с обратной связью по выходу
 * с размером синхропосылки 16 байт владеющий контекстом блочного шифра. Реализует
 * синхронный потоковый шифр. Определен в ГОСТ Р 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_ofb_new
 * - #kuznyechik_ofb_apply_keystream
 * - #kuznyechik_ofb_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #KuznyechikKey
 * - #KuznyechikNonce
 */
typedef struct ALIGNED(16) KuznyechikOfb {
  uint8_t p[208];
} KuznyechikOfb;

/**
 * Блочный режим с длиной блока 64 бит в режиме гаммирования с обратной связью по выходу
 * с размером синхропосылки 8 байт владеющий контекстом блочного шифра. Реализует
 * синхронный потоковый шифр. Определен в ГОСТ Р 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_ofb_new
 * - #magma_ofb_apply_keystream
 * - #magma_ofb_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #MagmaKey
 * - #MagmaNonce
 */
typedef struct ALIGNED(4) MagmaOfb {
  uint8_t p[52];
} MagmaOfb;

/**
 * Блочный режим с длиной блока 128 бит в режиме гаммирования с обратной связью по выходу
 * с размером синхропосылки 16 байт использующий ссылку на контекст блочного шифра. Реализует
 * синхронный потоковый шифр. Определен в ГОСТ Р 34.13-2015  и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #kuznyechik_ref_ofb_new
 * - #kuznyechik_ref_ofb_apply_keystream
 * - #kuznyechik_ref_ofb_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #Kuznyechik
 * - #KuznyechikNonce
 */
typedef struct ALIGNED(8) KuznyechikRefOfb {
  uint8_t p[48];
} KuznyechikRefOfb;

/**
 * Блочный режим с длиной блока 64 бит в режиме гаммирования с обратной связью по выходу
 * с размером синхропосылки 8 байт использующий ссылку на контекст блочного шифра. Реализует
 * синхронный потоковый шифр. Определен в ГОСТ Р 34.13-2015 и ГОСТ 34.13-2018.
 *
 * Набор функций для работы с шифром:
 *
 * - #magma_ref_ofb_new
 * - #magma_ref_ofb_apply_keystream
 * - #magma_ref_ofb_drop
 *
 * Набор типов для работы с блочным режимом:
 *
 * - #Magma
 * - #MagmaNonce
 */
typedef struct ALIGNED(8) MagmaRefOfb {
  uint8_t p[32];
} MagmaRefOfb;

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

/**
 * @brief Создать контекст блочного шифра #KuznyechikDec.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param dst — ненулевой указатель на память для хранения контекста шифра.
 * @param key — ненулевой указатель на ключ шифра.
 *
 * \see #KuznyechikDec
 */
void kuznyechik_dec_new(struct KuznyechikDec *dst,
                        const KuznyechikKey *key);

/**
 * @brief Создать контекст блочного шифра #KuznyechikDec из #KuznyechikEnc.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param dst — ненулевой указатель на память для хранения контекста шифра.
 * @param cipher — ненулевой указатель на корректный контекст #KuznyechikEnc.
 *
 * \see #KuznyechikDec
 */
void kuznyechik_dec_from_enc(struct KuznyechikDec *dst,
                             const struct KuznyechikEnc *enc_cipher);

/**
 * @brief Расшифровать блок данных.
 *
 * Расшифровывает блок расположенный по указателю `in_block`
 * с помощью заданного контекста #KuznyechikDec и зписывает результат
 * по указателю `out_block`.
 *
 * `in_block` и `out_block` должны быть либо равны, либо участки их памяти
 * не должны пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher — ненулевой указатель на корректный контекст #KuznyechikDec.
 * @param in_block — ненулевой указатель на входной блок.
 * @param out_block — ненулевой указатель на выходной блок.
 *
 * \see #KuznyechikDec
 */
void kuznyechik_dec_decrypt_block(const struct KuznyechikDec *cipher,
                                  const KuznyechikBlock *in_block,
                                  KuznyechikBlock *out_block);

/**
 * @brief Расшифровать массив блоков.
 *
 * Шифрует `len` блоков расположенных по указателю `in_blocks`
 * с помощью заданного контекста #KuznyechikDec и зписывает результат
 * в буфер по указателю `out_blocks`.
 *
 * `in_blocks` и `out_blocks` должны быть либо равны, либо участки памяти
 * `in_blocks..in_blocks + len` и `out_blocks..in_blocks + len` не должны
 * пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к
 * неопределенному поведению.
 *
 * @param cipher — ненулевой указатель на корректный контекст #KuznyechikDec.
 * @param in_blocks — ненулевой указатель на массив входных блоков.
 * @param out_blocks — ненулевой указатель на массив выходных блоков.
 * @param len — длина массива блоков (в блоках).
 *
 * \see #KuznyechikDec
 */
void kuznyechik_dec_decrypt_blocks(const struct KuznyechikDec *cipher,
                                   const KuznyechikBlock *in_blocks,
                                   KuznyechikBlock *out_blocks,
                                   uintptr_t len);

/**
 * @brief Расшифровать сообщение.
 *
 * Расшифровывает сообщение длиной с помощью заданного контекста #KuznyechikDec
 * используя режим простой замены определённый в ГОСТ Р 34.13-2015, после чего
 * пробует убрать дополнение. Результат записывается в выходной буфер по
 * указателю `out_buf`.
 *
 * Возвращает ошибку если полученное дополнение некорректно и если длина `len` не кратна размеру блока.
 *
 * `in_buf` и `out_buf` должны быть либо равны, либо участки памяти
 * `in_buf..in_buf + len` и `out_buf..out_buf + len` не должны пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на корректный контекст #KuznyechikDec.
 * @param in_buf — ненулевой указатель на входное сообщение.
 * @param out_buf — ненулевой указатель на выходной буфер.
 * @param len — длина входного сообщения и выходного буфера (в байтах).
 *
 * @returns Длину расшифрованного текста, в случае успеха или -1, в случае ошибки.
 *
 * \see #KuznyechikDec
 */
intptr_t kuznyechik_dec_ecb_decrypt_unpad(const struct KuznyechikDec *cipher,
                                          const uint8_t *in_buf,
                                          uint8_t *out_buf,
                                          uintptr_t len);

/**
 * @brief Уничтожить контекст #KuznyechikDec.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_dec_new или скопирован из существующего.
 * После вызова этой функции работать с этим контекстом нельзя, так как он перестает быть корректным.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher — ненулевой указатель на корректный контекст.
 *
 * \see #KuznyechikDec
 */
void kuznyechik_dec_drop(struct KuznyechikDec *cipher);

/**
 * @brief Создать контекст блочного шифра #KuznyechikEnc.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param dst — ненулевой указатель на память для хранения контекста шифра.
 * @param key — ненулевой указатель на ключ шифра.
 *
 * \see #KuznyechikEnc
 */
void kuznyechik_enc_new(struct KuznyechikEnc *dst,
                        const KuznyechikKey *key);

/**
 * @brief Зашифровать блок данных.
 *
 * Зашифровывает блок расположенный по указателю `in_block`
 * с помощью заданного контекста #KuznyechikEnc и зписывает результат
 * по указателю `out_block`.
 *
 * `in_block` и `out_block` должны быть либо равны, либо участки их памяти
 * не должны пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher — ненулевой указатель на корректный контекст #KuznyechikEnc.
 * @param in_block — ненулевой указатель на входной блок.
 * @param out_block — ненулевой указатель на выходной блок.
 *
 * \see #KuznyechikEnc
 */
void kuznyechik_enc_encrypt_block(const struct KuznyechikEnc *cipher,
                                  const KuznyechikBlock *in_block,
                                  KuznyechikBlock *out_block);

/**
 * @brief Зашифровать массив блоков.
 *
 * Шифрует `len` блоков по указателю `in_blocks` с помощью заданного
 * контекста #KuznyechikEnc и зписывает результат в буфер по указателю
 * `out_blocks`.
 *
 * `in_blocks` и `out_blocks` должны быть либо равны, либо участки памяти
 * `in_blocks..in_blocks + len` и `out_blocks..in_blocks + len` не должны
 * пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к
 * неопределенному поведению.
 *
 * @param cipher — ненулевой указатель на корректный контекст #KuznyechikEnc.
 * @param in_blocks — ненулевой указатель на массив входных блоков.
 * @param out_blocks — ненулевой указатель на массив выходных блоков.
 * @param len — длина массива блоков (в блоках).
 *
 * \see #KuznyechikEnc
 */
void kuznyechik_enc_encrypt_blocks(const struct KuznyechikEnc *cipher,
                                   const KuznyechikBlock *in_blocks,
                                   KuznyechikBlock *out_blocks,
                                   uintptr_t len);

/**
 * @brief Дополнить и зашифровать сообщение.
 *
 * Дополняет и зашифровывает сообщение длиной `in_len` байт по указателю
 * `buf_len` с помощью заданного контекста #KuznyechikEnc
 * используя режим простой замены определённый в ГОСТ Р 34.13-2015.
 * Результат записывается в выходной буфер по указателю `out_buf`.
 *
 * Возвращает ошибку в случае, когда длина получаемого шифротекста
 * больше `out_buf`.
 *
 * `in_buf` и `out_buf` должны быть либо равны, либо участки памяти
 * `in_buf..in_buf + in_len` и `out_buf..out_buf + out_len` не должны
 * пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на корректный контекст #KuznyechikEnc.
 * @param in_buf — ненулевой указатель на входное сообщение.
 * @param in_len — длина входного сообщения (в байтах).
 * @param out_buf — ненулевой указатель на выходной буфер.
 * @param out_len — длина выходного буфера (в байтах).
 *
 * @returns Длину шифротекста, в случае успеха или -1, в случае ошибки.
 *
 * \see #KuznyechikEnc
 */
intptr_t kuznyechik_enc_ecb_encrypt_pad(const struct KuznyechikEnc *cipher,
                                        const uint8_t *in_buf,
                                        uintptr_t in_len,
                                        uint8_t *out_buf,
                                        uintptr_t out_len);

/**
 * @brief Уничтожить контекст #KuznyechikEnc.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_enc_new или скопирован из существующего.
 * После вызова этой функции работать с этим контекстом нельзя, так как он перестает быть корректным.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher — ненулевой указатель на корректный контекст #KuznyechikEnc.
 *
 * \see #KuznyechikEnc
 */
void kuznyechik_enc_drop(struct KuznyechikEnc *cipher);

/**
 * @brief Создать контекст блочного шифра.
 *
 * Создает новый контекст блочного шифра.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param dst - ненулевой указатель на память для хранения контекста шифра.
 * @param key - ненулевой указатель на ключ шифра.
 *
 * \see #Magma
 */
void magma_new(struct Magma *dst,
               const MagmaKey *key);

/**
 * @brief Зашифровать блок данных.
 *
 * Зашифровывает блок расположенный по указателю `in_block`
 * с помощью заданного контекста #MagmaEnc и зписывает результат
 * по указателю `out_block`.
 *
 * `in_block` и `out_block` должны быть либо равны, либо участки их памяти
 * не должны пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher — ненулевой указатель на корректный контекст #MagmaEnc.
 * @param in_block — ненулевой указатель на входной блок.
 * @param out_block — ненулевой указатель на выходной блок.
 *
 * \see #Magma
 */
void magma_encrypt_block(const struct Magma *cipher,
                         const MagmaBlock *in_block,
                         MagmaBlock *out_block);

/**
 * @brief Расшифровать блок данных.
 *
 * Расшифровывает блок расположенный по указателю `in_block`
 * с помощью заданного контекста #MagmaDec и зписывает результат
 * по указателю `out_block`.
 *
 * `in_block` и `out_block` должны быть либо равны, либо участки их памяти
 * не должны пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher — ненулевой указатель на корректный контекст #MagmaDec.
 * @param in_block — ненулевой указатель на входной блок.
 * @param out_block — ненулевой указатель на выходной блок.
 *
 * \see #Magma
 */
void magma_decrypt_block(const struct Magma *cipher,
                         const MagmaBlock *in_block,
                         MagmaBlock *out_block);

/**
 * @brief Зашифровать массив блоков.
 *
 * Шифрует `len` блоков по указателю `in_blocks` с помощью заданного
 * контекста #MagmaEnc и зписывает результат в буфер по указателю
 * `out_blocks`.
 *
 * `in_blocks` и `out_blocks` должны быть либо равны, либо участки памяти
 * `in_blocks..in_blocks + len` и `out_blocks..in_blocks + len` не должны
 * пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к
 * неопределенному поведению.
 *
 * @param cipher — ненулевой указатель на корректный контекст #MagmaEnc.
 * @param in_blocks — ненулевой указатель на массив входных блоков.
 * @param out_blocks — ненулевой указатель на массив выходных блоков.
 * @param len — длина массива блоков (в блоках).
 *
 * \see #Magma
 */
void magma_encrypt_blocks(const struct Magma *cipher,
                          const MagmaBlock *in_blocks,
                          MagmaBlock *out_blocks,
                          uintptr_t len);

/**
 * @brief Расшифровать массив блоков.
 *
 * Шифрует `len` блоков расположенных по указателю `in_blocks`
 * с помощью заданного контекста #MagmaDec и зписывает результат
 * в буфер по указателю `out_blocks`.
 *
 * `in_blocks` и `out_blocks` должны быть либо равны, либо участки памяти
 * `in_blocks..in_blocks + len` и `out_blocks..in_blocks + len` не должны
 * пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к
 * неопределенному поведению.
 *
 * @param cipher — ненулевой указатель на корректный контекст #MagmaDec.
 * @param in_blocks — ненулевой указатель на массив входных блоков.
 * @param out_blocks — ненулевой указатель на массив выходных блоков.
 * @param len — длина массива блоков (в блоках).
 *
 * \see #Magma
 */
void magma_decrypt_blocks(const struct Magma *cipher,
                          const MagmaBlock *in_blocks,
                          MagmaBlock *out_blocks,
                          uintptr_t len);

/**
 * @brief Дополнить и зашифровать сообщение.
 *
 * Дополняет и зашифровывает сообщение длиной `in_len` байт по указателю
 * `buf_len` с помощью заданного контекста #MagmaEnc
 * используя режим простой замены определённый в ГОСТ Р 34.13-2015.
 * Результат записывается в выходной буфер по указателю `out_buf`.
 *
 * Возвращает ошибку в случае, когда длина получаемого шифротекста
 * больше `out_buf`.
 *
 * `in_buf` и `out_buf` должны быть либо равны, либо участки памяти
 * `in_buf..in_buf + in_len` и `out_buf..out_buf + out_len` не должны
 * пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на корректный контекст #MagmaEnc.
 * @param in_buf — ненулевой указатель на входное сообщение.
 * @param in_len — длина входного сообщения (в байтах).
 * @param out_buf — ненулевой указатель на выходной буфер.
 * @param out_len — длина выходного буфера (в байтах).
 *
 * @returns Длину шифротекста, в случае успеха или -1, в случае ошибки.
 *
 * \see #Magma
 */
intptr_t magma_ecb_encrypt_pad(const struct Magma *cipher,
                               const uint8_t *in_buf,
                               uintptr_t in_len,
                               uint8_t *out_buf,
                               uintptr_t out_len);

/**
 * @brief Расшифровать сообщение.
 *
 * Расшифровывает сообщение длиной с помощью заданного контекста #MagmaDec
 * используя режим простой замены определённый в ГОСТ Р 34.13-2015, после чего
 * пробует убрать дополнение. Результат записывается в выходной буфер по
 * указателю `out_buf`.
 *
 * Возвращает ошибку если полученное дополнение некорректно и если длина `len` не кратна размеру блока.
 *
 * `in_buf` и `out_buf` должны быть либо равны, либо участки памяти
 * `in_buf..in_buf + len` и `out_buf..out_buf + len` не должны пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на корректный контекст #MagmaDec.
 * @param in_buf — ненулевой указатель на входное сообщение.
 * @param out_buf — ненулевой указатель на выходной буфер.
 * @param len — длина входного сообщения и выходного буфера (в байтах).
 *
 * @returns Длину расшифрованного текста, в случае успеха или -1, в случае ошибки.
 *
 * \see #Magma
 */
intptr_t magma_ecb_decrypt_unpad(const struct Magma *cipher,
                                 const uint8_t *in_buf,
                                 uint8_t *out_buf,
                                 uintptr_t len);

/**
 * @brief Уничтожить контекст.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_new или скопирован из существующего.
 * После вызова этой функции работать с этим контекстом нельзя, так как он перестает быть корректным.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределенному поведению.
 *
 * @param cipher - ненулевой указатель на корректный контекст.
 *
 * \see #Magma
 */
void magma_drop(struct Magma *cipher);

/**
 * @brief Выработать 256-битный общий ключ используя ключи кривой `id-tc26-gost-3410-12-256-paramSetA`.
 *
 * Выработка общего ключа производится на основе ключа подписи стороны 1 и
 * ключа верификации подписи стороны 2. При условии использования одинакового `ukm`,
 * использование ключа подписи стороны 2 и ключа верификации подписи стороны 1
 * вернёт тот же самый общий ключ.
 *
 * Если в `ukm` передан нулевой указатель, то используется значение по
 * умолчанию равное единице. Если указатель не нулевой, то значение на
 * которое он указывает не должно быть равным нулю по модулю простого
 * числа эллиптической кривой. В случае передачи такого значения
 * возвращается ошибка. Если оба используемых ключа статичны
 * (т.е. используются неэфемерные ключи), то рекомендуется использовать
 * `ukm`, содержащий не менее 128 переменных бит.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * (Для параметра `ukm` нулевое значение допустимо.)
 *
 * @param sk — свой ключ формирования подписи.
 * @param vk — чужой ключ проверки подписи.
 * @param ukm — указатель на пользовательский ключевой материал (опционально).
 * @param out — результирующий общий ключ.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_256A_SignatureKey
 * \see #Ec2012_256A_VerificationKey
 */
intptr_t ec_2012_256a_vko256(const struct Ec2012_256A_SignatureKey *sk,
                             const struct Ec2012_256A_VerificationKey *vk,
                             const Ec256ScalarBytes *ukm,
                             Ec256ScalarBytes *key_dst);

/**
 * @brief Десериалировать ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetA`.
 *
 * Десериализует ключ подписи из представления определённого в
 * ГОСТ 34.10-2018 раздел 5.3 во внутреннее представление.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk_bytes — сериализованное представление ключа подписи.
 * @param sk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ подписи для кривой.
 *
 * \see #Ec2012_256A_SignatureKey
 */
intptr_t ec_2012_256a_deserialize_signature_key(const Ec256ScalarBytes *sk_bytes,
                                                struct Ec2012_256A_SignatureKey *sk_dst);

/**
 * @brief Cериалировать ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetA`.
 *
 * Cериализует ключ подписи из внутреннего представления в представление определённое
 * в ГОСТ 34.10-2018 раздел 5.3.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ подписи.
 * @param sk_bytes_dst — указатель на память в которую будет записан сериализованный ключ.
 *
 * \see #Ec2012_256A_SignatureKey
 */
void ec_2012_256a_serialize_signature_key(const struct Ec2012_256A_SignatureKey *sk,
                                          Ec256ScalarBytes *sk_bytes_dst);

/**
 * @brief Получить ключ проверки подписи из ключа формирования подписи.
 *
 * Получает ключ проверки подписи из ключа формирования подписи. Оба ключа принадлежат
 * к кривой `id-tc26-gost-3410-12-256-paramSetA`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param vk_dst — результирующий ключ проверки подписи.
 *
 * \see #Ec2012_256A_SignatureKey
 * \see #Ec2012_256A_VerificationKey
 */
void ec_2012_256a_get_verification_key(const struct Ec2012_256A_SignatureKey *sk,
                                       struct Ec2012_256A_VerificationKey *vk_dst);

/**
 * @brief Десериалировать ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetA`.
 *
 * Десериализует ключ проверки подписи из представления определённого в
 * ГОСТ 34.10-2018 раздел 5.3 во внутреннее
 * представление.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk_bytes — сериализованное представление ключа проверки подписи.
 * @param vk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ проверки подписи для кривой.
 *
 * \see #Ec2012_256A_VerificationKey
 */
intptr_t ec_2012_256a_deserialize_verification_key(const Ec256DoubleScalarBytes *vk_bytes,
                                                   struct Ec2012_256A_VerificationKey *vk_dst);

/**
 * @brief Cериалировать ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetA`.
 *
 * Cериализует ключ проверки подписи из внутреннего представления в представление
 * определённое в ГОСТ 34.10-2018 раздел 5.3.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk_bytes — сериализованное представление ключа проверки подписи.
 * @param vk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ проверки подписи для кривой.
 *
 * \see #Ec2012_256A_VerificationKey
 */
void ec_2012_256a_serialize_verification_key(const struct Ec2012_256A_VerificationKey *vk,
                                             Ec256DoubleScalarBytes *vk_bytes_dst);

/**
 * @brief Подписать хэш-код сообщения используя ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetA`.
 *
 * Формирует электронную подпись для хэш-код сообщения с помощью ключа подписи
 * принадлежащего `id-tc26-gost-3410-12-256-paramSetA`. Согласно ГОСТ 34.10-2018 хэш-код сообщения
 * должен быть вычислен с помощью функции Стрибог определённой в ГОСТ 34.11-2018.
 *
 * Функция возвращает ошибку если переданные `rand_bytes` не позволяют сформировать
 * корректную цифровую подпись. В таком случае данную функцию следует вызвать заново
 * с новым значением `rand_bytes`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param hash — хэш-код сообщения.
 * @param rand_bytes — набор чисел, сгенерированный случайным образом.
 * @param signature — результирующая подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_256A_SignatureKey
 */
intptr_t ec_2012_256a_dsa_sign_hash(const struct Ec2012_256A_SignatureKey *sk,
                                    const Streebog256Result *hash,
                                    const Ec256ScalarBytes *rand_bytes,
                                    Ec256DoubleScalarBytes *signature);

/**
 * @brief Подписать сообщения используя ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetA`.
 *
 * Формирует электронную подпись для сообщения с помощью ключа подписи
 * принадлежащего `id-tc26-gost-3410-12-256-paramSetA`.
 *
 * Функция возвращает ошибку если переданные `rand_bytes` не позволяют сформировать
 * корректную цифровую подпись. В таком случае данную функцию следует вызвать заново
 * с новым значением `rand_bytes`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param msg — указатель на сообщение.
 * @param msg_len — длина сообщения.
 * @param rand_bytes — набор чисел, сгенерированный случайным образом.
 * @param signature_dst — указатель на память в которую будет записана результирующая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_256A_SignatureKey
 */
intptr_t ec_2012_256a_dsa_sign_msg(const struct Ec2012_256A_SignatureKey *sk,
                                   const uint8_t *msg,
                                   uintptr_t msg_len,
                                   const Ec256ScalarBytes *rand_bytes,
                                   Ec256DoubleScalarBytes *signature_dst);

/**
 * @brief Проверить подпись хэш-кода сообщения используя ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetA`.
 *
 * Проверяет электронную подпись хэш-кода сообщения с помощью ключа проверки подписи
 * принадлежащего `id-tc26-gost-3410-12-256-paramSetA`. Согласно ГОСТ 34.10-2018 хэш-код сообщения
 * должен быть вычислен с помощью функции Стрибог определённой в ГОСТ 34.11-2018.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk — ключ проверки подписи.
 * @param hash — хэш-код сообщения.
 * @param signature — проверяемая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибочности цифровой подписи
 *
 * \see #Ec2012_256A_VerificationKey
 */
intptr_t ec_2012_256a_verify_hash_sig(const struct Ec2012_256A_VerificationKey *vk,
                                      const Streebog256Result *hash,
                                      const Ec256DoubleScalarBytes *signature);

/**
 * @brief Проверить подпись сообщения используя ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetA`.
 *
 * Проверяет электронную подпись сообщения с помощью ключа проверки подписи принадлежащего `id-tc26-gost-3410-12-256-paramSetA`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk — ключ проверки подписи.
 * @param msg — указатель на сообщение.
 * @param msg_len — длина сообщения.
 * @param signature — проверяемая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибочности цифровой подписи
 *
 * \see #Ec2012_256A_VerificationKey
 */
intptr_t ec_2012_256a_verify_msg_sig(const struct Ec2012_256A_VerificationKey *vk,
                                     const uint8_t *msg,
                                     uintptr_t msg_len,
                                     const Ec256DoubleScalarBytes *signature);

/**
 * @brief Уничтожить ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetA`.
 *
 * Очищает переданный ключ и обнуляет память.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование ключа после вызова данной функции приводит к неопределённому поведению.
 *
 * @param sk — ненулевой указатель на закрытый ключ
 *
 * \see #Ec2012_256A_SignatureKey
 */
void ec_2012_256a_secret_key_drop(struct Ec2012_256A_SignatureKey *sk);

/**
 * @brief Выработать 256-битный общий ключ используя ключи кривой `id-tc26-gost-3410-12-256-paramSetB`.
 *
 * Выработка общего ключа производится на основе ключа подписи стороны 1 и
 * ключа верификации подписи стороны 2. При условии использования одинакового `ukm`,
 * использование ключа подписи стороны 2 и ключа верификации подписи стороны 1
 * вернёт тот же самый общий ключ.
 *
 * Если в `ukm` передан нулевой указатель, то используется значение по
 * умолчанию равное единице. Если указатель не нулевой, то значение на
 * которое он указывает не должно быть равным нулю по модулю простого
 * числа эллиптической кривой. В случае передачи такого значения
 * возвращается ошибка. Если оба используемых ключа статичны
 * (т.е. используются неэфемерные ключи), то рекомендуется использовать
 * `ukm`, содержащий не менее 128 переменных бит.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * (Для параметра `ukm` нулевое значение допустимо.)
 *
 * @param sk — свой ключ формирования подписи.
 * @param vk — чужой ключ проверки подписи.
 * @param ukm — указатель на пользовательский ключевой материал (опционально).
 * @param out — результирующий общий ключ.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_256B_SignatureKey
 * \see #Ec2012_256B_VerificationKey
 */
intptr_t ec_2012_256b_vko256(const struct Ec2012_256B_SignatureKey *sk,
                             const struct Ec2012_256B_VerificationKey *vk,
                             const Ec256ScalarBytes *ukm,
                             Ec256ScalarBytes *key_dst);

/**
 * @brief Десериалировать ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetB`.
 *
 * Десериализует ключ подписи из представления определённого в
 * ГОСТ 34.10-2018 раздел 5.3 во внутреннее представление.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk_bytes — сериализованное представление ключа подписи.
 * @param sk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ подписи для кривой.
 *
 * \see #Ec2012_256B_SignatureKey
 */
intptr_t ec_2012_256b_deserialize_signature_key(const Ec256ScalarBytes *sk_bytes,
                                                struct Ec2012_256B_SignatureKey *sk_dst);

/**
 * @brief Cериалировать ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetB`.
 *
 * Cериализует ключ подписи из внутреннего представления в представление определённое
 * в ГОСТ 34.10-2018 раздел 5.3.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ подписи.
 * @param sk_bytes_dst — указатель на память в которую будет записан сериализованный ключ.
 *
 * \see #Ec2012_256B_SignatureKey
 */
void ec_2012_256b_serialize_signature_key(const struct Ec2012_256B_SignatureKey *sk,
                                          Ec256ScalarBytes *sk_bytes_dst);

/**
 * @brief Получить ключ проверки подписи из ключа формирования подписи.
 *
 * Получает ключ проверки подписи из ключа формирования подписи. Оба ключа принадлежат
 * к кривой `id-tc26-gost-3410-12-256-paramSetB`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param vk_dst — результирующий ключ проверки подписи.
 *
 * \see #Ec2012_256B_SignatureKey
 * \see #Ec2012_256B_VerificationKey
 */
void ec_2012_256b_get_verification_key(const struct Ec2012_256B_SignatureKey *sk,
                                       struct Ec2012_256B_VerificationKey *vk_dst);

/**
 * @brief Десериалировать ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetB`.
 *
 * Десериализует ключ проверки подписи из представления определённого в
 * ГОСТ 34.10-2018 раздел 5.3 во внутреннее
 * представление.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk_bytes — сериализованное представление ключа проверки подписи.
 * @param vk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ проверки подписи для кривой.
 *
 * \see #Ec2012_256B_VerificationKey
 */
intptr_t ec_2012_256b_deserialize_verification_key(const Ec256DoubleScalarBytes *vk_bytes,
                                                   struct Ec2012_256B_VerificationKey *vk_dst);

/**
 * @brief Cериалировать ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetB`.
 *
 * Cериализует ключ проверки подписи из внутреннего представления в представление
 * определённое в ГОСТ 34.10-2018 раздел 5.3.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk_bytes — сериализованное представление ключа проверки подписи.
 * @param vk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ проверки подписи для кривой.
 *
 * \see #Ec2012_256B_VerificationKey
 */
void ec_2012_256b_serialize_verification_key(const struct Ec2012_256B_VerificationKey *vk,
                                             Ec256DoubleScalarBytes *vk_bytes_dst);

/**
 * @brief Подписать хэш-код сообщения используя ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetB`.
 *
 * Формирует электронную подпись для хэш-код сообщения с помощью ключа подписи
 * принадлежащего `id-tc26-gost-3410-12-256-paramSetB`. Согласно ГОСТ 34.10-2018 хэш-код сообщения
 * должен быть вычислен с помощью функции Стрибог определённой в ГОСТ 34.11-2018.
 *
 * Функция возвращает ошибку если переданные `rand_bytes` не позволяют сформировать
 * корректную цифровую подпись. В таком случае данную функцию следует вызвать заново
 * с новым значением `rand_bytes`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param hash — хэш-код сообщения.
 * @param rand_bytes — набор чисел, сгенерированный случайным образом.
 * @param signature — результирующая подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_256B_SignatureKey
 */
intptr_t ec_2012_256b_dsa_sign_hash(const struct Ec2012_256B_SignatureKey *sk,
                                    const Streebog256Result *hash,
                                    const Ec256ScalarBytes *rand_bytes,
                                    Ec256DoubleScalarBytes *signature);

/**
 * @brief Подписать сообщения используя ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetB`.
 *
 * Формирует электронную подпись для сообщения с помощью ключа подписи
 * принадлежащего `id-tc26-gost-3410-12-256-paramSetB`.
 *
 * Функция возвращает ошибку если переданные `rand_bytes` не позволяют сформировать
 * корректную цифровую подпись. В таком случае данную функцию следует вызвать заново
 * с новым значением `rand_bytes`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param msg — указатель на сообщение.
 * @param msg_len — длина сообщения.
 * @param rand_bytes — набор чисел, сгенерированный случайным образом.
 * @param signature_dst — указатель на память в которую будет записана результирующая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_256B_SignatureKey
 */
intptr_t ec_2012_256b_dsa_sign_msg(const struct Ec2012_256B_SignatureKey *sk,
                                   const uint8_t *msg,
                                   uintptr_t msg_len,
                                   const Ec256ScalarBytes *rand_bytes,
                                   Ec256DoubleScalarBytes *signature_dst);

/**
 * @brief Проверить подпись хэш-кода сообщения используя ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetB`.
 *
 * Проверяет электронную подпись хэш-кода сообщения с помощью ключа проверки подписи
 * принадлежащего `id-tc26-gost-3410-12-256-paramSetB`. Согласно ГОСТ 34.10-2018 хэш-код сообщения
 * должен быть вычислен с помощью функции Стрибог определённой в ГОСТ 34.11-2018.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk — ключ проверки подписи.
 * @param hash — хэш-код сообщения.
 * @param signature — проверяемая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибочности цифровой подписи
 *
 * \see #Ec2012_256B_VerificationKey
 */
intptr_t ec_2012_256b_verify_hash_sig(const struct Ec2012_256B_VerificationKey *vk,
                                      const Streebog256Result *hash,
                                      const Ec256DoubleScalarBytes *signature);

/**
 * @brief Проверить подпись сообщения используя ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetB`.
 *
 * Проверяет электронную подпись сообщения с помощью ключа проверки подписи принадлежащего `id-tc26-gost-3410-12-256-paramSetB`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk — ключ проверки подписи.
 * @param msg — указатель на сообщение.
 * @param msg_len — длина сообщения.
 * @param signature — проверяемая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибочности цифровой подписи
 *
 * \see #Ec2012_256B_VerificationKey
 */
intptr_t ec_2012_256b_verify_msg_sig(const struct Ec2012_256B_VerificationKey *vk,
                                     const uint8_t *msg,
                                     uintptr_t msg_len,
                                     const Ec256DoubleScalarBytes *signature);

/**
 * @brief Уничтожить ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetB`.
 *
 * Очищает переданный ключ и обнуляет память.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование ключа после вызова данной функции приводит к неопределённому поведению.
 *
 * @param sk — ненулевой указатель на закрытый ключ
 *
 * \see #Ec2012_256B_SignatureKey
 */
void ec_2012_256b_secret_key_drop(struct Ec2012_256B_SignatureKey *sk);

/**
 * @brief Выработать 256-битный общий ключ используя ключи кривой `id-tc26-gost-3410-12-256-paramSetC`.
 *
 * Выработка общего ключа производится на основе ключа подписи стороны 1 и
 * ключа верификации подписи стороны 2. При условии использования одинакового `ukm`,
 * использование ключа подписи стороны 2 и ключа верификации подписи стороны 1
 * вернёт тот же самый общий ключ.
 *
 * Если в `ukm` передан нулевой указатель, то используется значение по
 * умолчанию равное единице. Если указатель не нулевой, то значение на
 * которое он указывает не должно быть равным нулю по модулю простого
 * числа эллиптической кривой. В случае передачи такого значения
 * возвращается ошибка. Если оба используемых ключа статичны
 * (т.е. используются неэфемерные ключи), то рекомендуется использовать
 * `ukm`, содержащий не менее 128 переменных бит.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * (Для параметра `ukm` нулевое значение допустимо.)
 *
 * @param sk — свой ключ формирования подписи.
 * @param vk — чужой ключ проверки подписи.
 * @param ukm — указатель на пользовательский ключевой материал (опционально).
 * @param out — результирующий общий ключ.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_256C_SignatureKey
 * \see #Ec2012_256C_VerificationKey
 */
intptr_t ec_2012_256c_vko256(const struct Ec2012_256C_SignatureKey *sk,
                             const struct Ec2012_256C_VerificationKey *vk,
                             const Ec256ScalarBytes *ukm,
                             Ec256ScalarBytes *key_dst);

/**
 * @brief Десериалировать ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetC`.
 *
 * Десериализует ключ подписи из представления определённого в
 * ГОСТ 34.10-2018 раздел 5.3 во внутреннее представление.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk_bytes — сериализованное представление ключа подписи.
 * @param sk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ подписи для кривой.
 *
 * \see #Ec2012_256C_SignatureKey
 */
intptr_t ec_2012_256c_deserialize_signature_key(const Ec256ScalarBytes *sk_bytes,
                                                struct Ec2012_256C_SignatureKey *sk_dst);

/**
 * @brief Cериалировать ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetC`.
 *
 * Cериализует ключ подписи из внутреннего представления в представление определённое
 * в ГОСТ 34.10-2018 раздел 5.3.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ подписи.
 * @param sk_bytes_dst — указатель на память в которую будет записан сериализованный ключ.
 *
 * \see #Ec2012_256C_SignatureKey
 */
void ec_2012_256c_serialize_signature_key(const struct Ec2012_256C_SignatureKey *sk,
                                          Ec256ScalarBytes *sk_bytes_dst);

/**
 * @brief Получить ключ проверки подписи из ключа формирования подписи.
 *
 * Получает ключ проверки подписи из ключа формирования подписи. Оба ключа принадлежат
 * к кривой `id-tc26-gost-3410-12-256-paramSetC`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param vk_dst — результирующий ключ проверки подписи.
 *
 * \see #Ec2012_256C_SignatureKey
 * \see #Ec2012_256C_VerificationKey
 */
void ec_2012_256c_get_verification_key(const struct Ec2012_256C_SignatureKey *sk,
                                       struct Ec2012_256C_VerificationKey *vk_dst);

/**
 * @brief Десериалировать ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetC`.
 *
 * Десериализует ключ проверки подписи из представления определённого в
 * ГОСТ 34.10-2018 раздел 5.3 во внутреннее
 * представление.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk_bytes — сериализованное представление ключа проверки подписи.
 * @param vk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ проверки подписи для кривой.
 *
 * \see #Ec2012_256C_VerificationKey
 */
intptr_t ec_2012_256c_deserialize_verification_key(const Ec256DoubleScalarBytes *vk_bytes,
                                                   struct Ec2012_256C_VerificationKey *vk_dst);

/**
 * @brief Cериалировать ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetC`.
 *
 * Cериализует ключ проверки подписи из внутреннего представления в представление
 * определённое в ГОСТ 34.10-2018 раздел 5.3.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk_bytes — сериализованное представление ключа проверки подписи.
 * @param vk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ проверки подписи для кривой.
 *
 * \see #Ec2012_256C_VerificationKey
 */
void ec_2012_256c_serialize_verification_key(const struct Ec2012_256C_VerificationKey *vk,
                                             Ec256DoubleScalarBytes *vk_bytes_dst);

/**
 * @brief Подписать хэш-код сообщения используя ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetC`.
 *
 * Формирует электронную подпись для хэш-код сообщения с помощью ключа подписи
 * принадлежащего `id-tc26-gost-3410-12-256-paramSetC`. Согласно ГОСТ 34.10-2018 хэш-код сообщения
 * должен быть вычислен с помощью функции Стрибог определённой в ГОСТ 34.11-2018.
 *
 * Функция возвращает ошибку если переданные `rand_bytes` не позволяют сформировать
 * корректную цифровую подпись. В таком случае данную функцию следует вызвать заново
 * с новым значением `rand_bytes`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param hash — хэш-код сообщения.
 * @param rand_bytes — набор чисел, сгенерированный случайным образом.
 * @param signature — результирующая подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_256C_SignatureKey
 */
intptr_t ec_2012_256c_dsa_sign_hash(const struct Ec2012_256C_SignatureKey *sk,
                                    const Streebog256Result *hash,
                                    const Ec256ScalarBytes *rand_bytes,
                                    Ec256DoubleScalarBytes *signature);

/**
 * @brief Подписать сообщения используя ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetC`.
 *
 * Формирует электронную подпись для сообщения с помощью ключа подписи
 * принадлежащего `id-tc26-gost-3410-12-256-paramSetC`.
 *
 * Функция возвращает ошибку если переданные `rand_bytes` не позволяют сформировать
 * корректную цифровую подпись. В таком случае данную функцию следует вызвать заново
 * с новым значением `rand_bytes`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param msg — указатель на сообщение.
 * @param msg_len — длина сообщения.
 * @param rand_bytes — набор чисел, сгенерированный случайным образом.
 * @param signature_dst — указатель на память в которую будет записана результирующая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_256C_SignatureKey
 */
intptr_t ec_2012_256c_dsa_sign_msg(const struct Ec2012_256C_SignatureKey *sk,
                                   const uint8_t *msg,
                                   uintptr_t msg_len,
                                   const Ec256ScalarBytes *rand_bytes,
                                   Ec256DoubleScalarBytes *signature_dst);

/**
 * @brief Проверить подпись хэш-кода сообщения используя ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetC`.
 *
 * Проверяет электронную подпись хэш-кода сообщения с помощью ключа проверки подписи
 * принадлежащего `id-tc26-gost-3410-12-256-paramSetC`. Согласно ГОСТ 34.10-2018 хэш-код сообщения
 * должен быть вычислен с помощью функции Стрибог определённой в ГОСТ 34.11-2018.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk — ключ проверки подписи.
 * @param hash — хэш-код сообщения.
 * @param signature — проверяемая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибочности цифровой подписи
 *
 * \see #Ec2012_256C_VerificationKey
 */
intptr_t ec_2012_256c_verify_hash_sig(const struct Ec2012_256C_VerificationKey *vk,
                                      const Streebog256Result *hash,
                                      const Ec256DoubleScalarBytes *signature);

/**
 * @brief Проверить подпись сообщения используя ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetC`.
 *
 * Проверяет электронную подпись сообщения с помощью ключа проверки подписи принадлежащего `id-tc26-gost-3410-12-256-paramSetC`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk — ключ проверки подписи.
 * @param msg — указатель на сообщение.
 * @param msg_len — длина сообщения.
 * @param signature — проверяемая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибочности цифровой подписи
 *
 * \see #Ec2012_256C_VerificationKey
 */
intptr_t ec_2012_256c_verify_msg_sig(const struct Ec2012_256C_VerificationKey *vk,
                                     const uint8_t *msg,
                                     uintptr_t msg_len,
                                     const Ec256DoubleScalarBytes *signature);

/**
 * @brief Уничтожить ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetC`.
 *
 * Очищает переданный ключ и обнуляет память.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование ключа после вызова данной функции приводит к неопределённому поведению.
 *
 * @param sk — ненулевой указатель на закрытый ключ
 *
 * \see #Ec2012_256C_SignatureKey
 */
void ec_2012_256c_secret_key_drop(struct Ec2012_256C_SignatureKey *sk);

/**
 * @brief Выработать 256-битный общий ключ используя ключи кривой `id-tc26-gost-3410-12-256-paramSetD`.
 *
 * Выработка общего ключа производится на основе ключа подписи стороны 1 и
 * ключа верификации подписи стороны 2. При условии использования одинакового `ukm`,
 * использование ключа подписи стороны 2 и ключа верификации подписи стороны 1
 * вернёт тот же самый общий ключ.
 *
 * Если в `ukm` передан нулевой указатель, то используется значение по
 * умолчанию равное единице. Если указатель не нулевой, то значение на
 * которое он указывает не должно быть равным нулю по модулю простого
 * числа эллиптической кривой. В случае передачи такого значения
 * возвращается ошибка. Если оба используемых ключа статичны
 * (т.е. используются неэфемерные ключи), то рекомендуется использовать
 * `ukm`, содержащий не менее 128 переменных бит.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * (Для параметра `ukm` нулевое значение допустимо.)
 *
 * @param sk — свой ключ формирования подписи.
 * @param vk — чужой ключ проверки подписи.
 * @param ukm — указатель на пользовательский ключевой материал (опционально).
 * @param out — результирующий общий ключ.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_256D_SignatureKey
 * \see #Ec2012_256D_VerificationKey
 */
intptr_t ec_2012_256d_vko256(const struct Ec2012_256D_SignatureKey *sk,
                             const struct Ec2012_256D_VerificationKey *vk,
                             const Ec256ScalarBytes *ukm,
                             Ec256ScalarBytes *key_dst);

/**
 * @brief Десериалировать ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetD`.
 *
 * Десериализует ключ подписи из представления определённого в
 * ГОСТ 34.10-2018 раздел 5.3 во внутреннее представление.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk_bytes — сериализованное представление ключа подписи.
 * @param sk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ подписи для кривой.
 *
 * \see #Ec2012_256D_SignatureKey
 */
intptr_t ec_2012_256d_deserialize_signature_key(const Ec256ScalarBytes *sk_bytes,
                                                struct Ec2012_256D_SignatureKey *sk_dst);

/**
 * @brief Cериалировать ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetD`.
 *
 * Cериализует ключ подписи из внутреннего представления в представление определённое
 * в ГОСТ 34.10-2018 раздел 5.3.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ подписи.
 * @param sk_bytes_dst — указатель на память в которую будет записан сериализованный ключ.
 *
 * \see #Ec2012_256D_SignatureKey
 */
void ec_2012_256d_serialize_signature_key(const struct Ec2012_256D_SignatureKey *sk,
                                          Ec256ScalarBytes *sk_bytes_dst);

/**
 * @brief Получить ключ проверки подписи из ключа формирования подписи.
 *
 * Получает ключ проверки подписи из ключа формирования подписи. Оба ключа принадлежат
 * к кривой `id-tc26-gost-3410-12-256-paramSetD`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param vk_dst — результирующий ключ проверки подписи.
 *
 * \see #Ec2012_256D_SignatureKey
 * \see #Ec2012_256D_VerificationKey
 */
void ec_2012_256d_get_verification_key(const struct Ec2012_256D_SignatureKey *sk,
                                       struct Ec2012_256D_VerificationKey *vk_dst);

/**
 * @brief Десериалировать ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetD`.
 *
 * Десериализует ключ проверки подписи из представления определённого в
 * ГОСТ 34.10-2018 раздел 5.3 во внутреннее
 * представление.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk_bytes — сериализованное представление ключа проверки подписи.
 * @param vk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ проверки подписи для кривой.
 *
 * \see #Ec2012_256D_VerificationKey
 */
intptr_t ec_2012_256d_deserialize_verification_key(const Ec256DoubleScalarBytes *vk_bytes,
                                                   struct Ec2012_256D_VerificationKey *vk_dst);

/**
 * @brief Cериалировать ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetD`.
 *
 * Cериализует ключ проверки подписи из внутреннего представления в представление
 * определённое в ГОСТ 34.10-2018 раздел 5.3.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk_bytes — сериализованное представление ключа проверки подписи.
 * @param vk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ проверки подписи для кривой.
 *
 * \see #Ec2012_256D_VerificationKey
 */
void ec_2012_256d_serialize_verification_key(const struct Ec2012_256D_VerificationKey *vk,
                                             Ec256DoubleScalarBytes *vk_bytes_dst);

/**
 * @brief Подписать хэш-код сообщения используя ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetD`.
 *
 * Формирует электронную подпись для хэш-код сообщения с помощью ключа подписи
 * принадлежащего `id-tc26-gost-3410-12-256-paramSetD`. Согласно ГОСТ 34.10-2018 хэш-код сообщения
 * должен быть вычислен с помощью функции Стрибог определённой в ГОСТ 34.11-2018.
 *
 * Функция возвращает ошибку если переданные `rand_bytes` не позволяют сформировать
 * корректную цифровую подпись. В таком случае данную функцию следует вызвать заново
 * с новым значением `rand_bytes`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param hash — хэш-код сообщения.
 * @param rand_bytes — набор чисел, сгенерированный случайным образом.
 * @param signature — результирующая подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_256D_SignatureKey
 */
intptr_t ec_2012_256d_dsa_sign_hash(const struct Ec2012_256D_SignatureKey *sk,
                                    const Streebog256Result *hash,
                                    const Ec256ScalarBytes *rand_bytes,
                                    Ec256DoubleScalarBytes *signature);

/**
 * @brief Подписать сообщения используя ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetD`.
 *
 * Формирует электронную подпись для сообщения с помощью ключа подписи
 * принадлежащего `id-tc26-gost-3410-12-256-paramSetD`.
 *
 * Функция возвращает ошибку если переданные `rand_bytes` не позволяют сформировать
 * корректную цифровую подпись. В таком случае данную функцию следует вызвать заново
 * с новым значением `rand_bytes`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param msg — указатель на сообщение.
 * @param msg_len — длина сообщения.
 * @param rand_bytes — набор чисел, сгенерированный случайным образом.
 * @param signature_dst — указатель на память в которую будет записана результирующая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_256D_SignatureKey
 */
intptr_t ec_2012_256d_dsa_sign_msg(const struct Ec2012_256D_SignatureKey *sk,
                                   const uint8_t *msg,
                                   uintptr_t msg_len,
                                   const Ec256ScalarBytes *rand_bytes,
                                   Ec256DoubleScalarBytes *signature_dst);

/**
 * @brief Проверить подпись хэш-кода сообщения используя ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetD`.
 *
 * Проверяет электронную подпись хэш-кода сообщения с помощью ключа проверки подписи
 * принадлежащего `id-tc26-gost-3410-12-256-paramSetD`. Согласно ГОСТ 34.10-2018 хэш-код сообщения
 * должен быть вычислен с помощью функции Стрибог определённой в ГОСТ 34.11-2018.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk — ключ проверки подписи.
 * @param hash — хэш-код сообщения.
 * @param signature — проверяемая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибочности цифровой подписи
 *
 * \see #Ec2012_256D_VerificationKey
 */
intptr_t ec_2012_256d_verify_hash_sig(const struct Ec2012_256D_VerificationKey *vk,
                                      const Streebog256Result *hash,
                                      const Ec256DoubleScalarBytes *signature);

/**
 * @brief Проверить подпись сообщения используя ключ проверки подписи кривой `id-tc26-gost-3410-12-256-paramSetD`.
 *
 * Проверяет электронную подпись сообщения с помощью ключа проверки подписи принадлежащего `id-tc26-gost-3410-12-256-paramSetD`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk — ключ проверки подписи.
 * @param msg — указатель на сообщение.
 * @param msg_len — длина сообщения.
 * @param signature — проверяемая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибочности цифровой подписи
 *
 * \see #Ec2012_256D_VerificationKey
 */
intptr_t ec_2012_256d_verify_msg_sig(const struct Ec2012_256D_VerificationKey *vk,
                                     const uint8_t *msg,
                                     uintptr_t msg_len,
                                     const Ec256DoubleScalarBytes *signature);

/**
 * @brief Уничтожить ключ подписи кривой `id-tc26-gost-3410-12-256-paramSetD`.
 *
 * Очищает переданный ключ и обнуляет память.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование ключа после вызова данной функции приводит к неопределённому поведению.
 *
 * @param sk — ненулевой указатель на закрытый ключ
 *
 * \see #Ec2012_256D_SignatureKey
 */
void ec_2012_256d_secret_key_drop(struct Ec2012_256D_SignatureKey *sk);

/**
 * @brief Выработать 256-битный общий ключ используя ключи кривой `id-tc26-gost-3410-12-512-paramSetA`.
 *
 * Выработка общего ключа производится на основе ключа подписи стороны 1 и
 * ключа верификации подписи стороны 2. При условии использования одинакового `ukm`,
 * использование ключа подписи стороны 2 и ключа верификации подписи стороны 1
 * вернёт тот же самый общий ключ.
 *
 * Если в `ukm` передан нулевой указатель, то используется значение по
 * умолчанию равное единице. Если указатель не нулевой, то значение на
 * которое он указывает не должно быть равным нулю по модулю простого
 * числа эллиптической кривой. В случае передачи такого значения
 * возвращается ошибка. Если оба используемых ключа статичны
 * (т.е. используются неэфемерные ключи), то рекомендуется использовать
 * `ukm`, содержащий не менее 128 переменных бит.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * (Для параметра `ukm` нулевое значение допустимо.)
 *
 * @param sk — свой ключ формирования подписи.
 * @param vk — чужой ключ проверки подписи.
 * @param ukm — указатель на пользовательский ключевой материал (опционально).
 * @param out — результирующий общий ключ.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_512A_SignatureKey
 * \see #Ec2012_512A_VerificationKey
 */
intptr_t ec_2012_512a_vko256(const struct Ec2012_512A_SignatureKey *sk,
                             const struct Ec2012_512A_VerificationKey *vk,
                             const Ec512ScalarBytes *ukm,
                             Ec256ScalarBytes *key_dst);

/**
 * @brief Выработать 512-битный общий ключ используя ключи кривой `id-tc26-gost-3410-12-512-paramSetA`.
 *
 * Выработка общего ключа производится на основе ключа подписи стороны 1 и
 * ключа верификации подписи стороны 2. При условии использования одинакового `ukm`,
 * использование ключа подписи стороны 2 и ключа верификации подписи стороны 1
 * вернёт тот же самый общий ключ.
 *
 * Если в `ukm` передан нулевой указатель, то используется значение по
 * умолчанию равное единице. Если указатель не нулевой, то значение на
 * которое он указывает не должно быть равным нулю по модулю простого
 * числа эллиптической кривой. В случае передачи такого значения
 * возвращается ошибка. Если оба используемых ключа статичны
 * (т.е. используются неэфемерные ключи), то рекомендуется использовать
 * `ukm`, содержащий не менее 128 переменных бит.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * (Для параметра `ukm` нулевое значение допустимо.)
 *
 * @param sk — свой ключ формирования подписи.
 * @param vk — чужой ключ проверки подписи.
 * @param ukm — указатель на пользовательский ключевой материал (опционально).
 * @param out — результирующий общий ключ.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_512A_SignatureKey
 * \see #Ec2012_512A_VerificationKey
 */
intptr_t ec_2012_512a_vko512(const struct Ec2012_512A_SignatureKey *sk,
                             const struct Ec2012_512A_VerificationKey *pk,
                             const Ec512ScalarBytes *ukm,
                             Ec512ScalarBytes *key_dst);

/**
 * @brief Десериалировать ключ подписи кривой `id-tc26-gost-3410-12-512-paramSetA`.
 *
 * Десериализует ключ подписи из представления определённого в
 * ГОСТ 34.10-2018 раздел 5.3 во внутреннее представление.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk_bytes — сериализованное представление ключа подписи.
 * @param sk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ подписи для кривой.
 *
 * \see #Ec2012_512A_SignatureKey
 */
intptr_t ec_2012_512a_deserialize_signature_key(const Ec512ScalarBytes *sk_bytes,
                                                struct Ec2012_512A_SignatureKey *sk_dst);

/**
 * @brief Cериалировать ключ подписи кривой `id-tc26-gost-3410-12-512-paramSetA`.
 *
 * Cериализует ключ подписи из внутреннего представления в представление определённое
 * в ГОСТ 34.10-2018 раздел 5.3.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ подписи.
 * @param sk_bytes_dst — указатель на память в которую будет записан сериализованный ключ.
 *
 * \see #Ec2012_512A_SignatureKey
 */
void ec_2012_512a_serialize_signature_key(const struct Ec2012_512A_SignatureKey *sk,
                                          Ec512ScalarBytes *sk_bytes_dst);

/**
 * @brief Получить ключ проверки подписи из ключа формирования подписи.
 *
 * Получает ключ проверки подписи из ключа формирования подписи. Оба ключа принадлежат
 * к кривой `id-tc26-gost-3410-12-512-paramSetA`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param vk_dst — результирующий ключ проверки подписи.
 *
 * \see #Ec2012_512A_SignatureKey
 * \see #Ec2012_512A_VerificationKey
 */
void ec_2012_512a_get_verification_key(const struct Ec2012_512A_SignatureKey *sk,
                                       struct Ec2012_512A_VerificationKey *vk_dst);

/**
 * @brief Десериалировать ключ проверки подписи кривой `id-tc26-gost-3410-12-512-paramSetA`.
 *
 * Десериализует ключ проверки подписи из представления определённого в
 * ГОСТ 34.10-2018 раздел 5.3 во внутреннее
 * представление.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk_bytes — сериализованное представление ключа проверки подписи.
 * @param vk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ проверки подписи для кривой.
 *
 * \see #Ec2012_512A_VerificationKey
 */
intptr_t ec_2012_512a_deserialize_verification_key(const Ec512DoubleScalarBytes *vk_bytes,
                                                   struct Ec2012_512A_VerificationKey *vk_dst);

/**
 * @brief Cериалировать ключ проверки подписи кривой `id-tc26-gost-3410-12-512-paramSetA`.
 *
 * Cериализует ключ проверки подписи из внутреннего представления в представление
 * определённое в ГОСТ 34.10-2018 раздел 5.3.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk_bytes — сериализованное представление ключа проверки подписи.
 * @param vk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ проверки подписи для кривой.
 *
 * \see #Ec2012_512A_VerificationKey
 */
void ec_2012_512a_serialize_verification_key(const struct Ec2012_512A_VerificationKey *vk,
                                             Ec512DoubleScalarBytes *vk_bytes_dst);

/**
 * @brief Подписать хэш-код сообщения используя ключ подписи кривой `id-tc26-gost-3410-12-512-paramSetA`.
 *
 * Формирует электронную подпись для хэш-код сообщения с помощью ключа подписи
 * принадлежащего `id-tc26-gost-3410-12-512-paramSetA`. Согласно ГОСТ 34.10-2018 хэш-код сообщения
 * должен быть вычислен с помощью функции Стрибог определённой в ГОСТ 34.11-2018.
 *
 * Функция возвращает ошибку если переданные `rand_bytes` не позволяют сформировать
 * корректную цифровую подпись. В таком случае данную функцию следует вызвать заново
 * с новым значением `rand_bytes`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param hash — хэш-код сообщения.
 * @param rand_bytes — набор чисел, сгенерированный случайным образом.
 * @param signature — результирующая подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_512A_SignatureKey
 */
intptr_t ec_2012_512a_dsa_sign_hash(const struct Ec2012_512A_SignatureKey *sk,
                                    const Streebog512Result *hash,
                                    const Ec512ScalarBytes *rand_bytes,
                                    Ec512DoubleScalarBytes *signature);

/**
 * @brief Подписать сообщения используя ключ подписи кривой `id-tc26-gost-3410-12-512-paramSetA`.
 *
 * Формирует электронную подпись для сообщения с помощью ключа подписи
 * принадлежащего `id-tc26-gost-3410-12-512-paramSetA`.
 *
 * Функция возвращает ошибку если переданные `rand_bytes` не позволяют сформировать
 * корректную цифровую подпись. В таком случае данную функцию следует вызвать заново
 * с новым значением `rand_bytes`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param msg — указатель на сообщение.
 * @param msg_len — длина сообщения.
 * @param rand_bytes — набор чисел, сгенерированный случайным образом.
 * @param signature_dst — указатель на память в которую будет записана результирующая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_512A_SignatureKey
 */
intptr_t ec_2012_512a_dsa_sign_msg(const struct Ec2012_512A_SignatureKey *sk,
                                   const uint8_t *msg,
                                   uintptr_t msg_len,
                                   const Ec512ScalarBytes *rand_bytes,
                                   Ec512DoubleScalarBytes *signature_dst);

/**
 * @brief Проверить подпись хэш-кода сообщения используя ключ проверки подписи кривой `id-tc26-gost-3410-12-512-paramSetA`.
 *
 * Проверяет электронную подпись хэш-кода сообщения с помощью ключа проверки подписи
 * принадлежащего `id-tc26-gost-3410-12-512-paramSetA`. Согласно ГОСТ 34.10-2018 хэш-код сообщения
 * должен быть вычислен с помощью функции Стрибог определённой в ГОСТ 34.11-2018.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk — ключ проверки подписи.
 * @param hash — хэш-код сообщения.
 * @param signature — проверяемая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибочности цифровой подписи
 *
 * \see #Ec2012_512A_VerificationKey
 */
intptr_t ec_2012_512a_verify_hash_sig(const struct Ec2012_512A_VerificationKey *vk,
                                      const Streebog512Result *hash,
                                      const Ec512DoubleScalarBytes *signature);

/**
 * @brief Проверить подпись сообщения используя ключ проверки подписи кривой `id-tc26-gost-3410-12-512-paramSetA`.
 *
 * Проверяет электронную подпись сообщения с помощью ключа проверки подписи принадлежащего `id-tc26-gost-3410-12-512-paramSetA`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk — ключ проверки подписи.
 * @param msg — указатель на сообщение.
 * @param msg_len — длина сообщения.
 * @param signature — проверяемая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибочности цифровой подписи
 *
 * \see #Ec2012_512A_VerificationKey
 */
intptr_t ec_2012_512a_verify_msg_sig(const struct Ec2012_512A_VerificationKey *vk,
                                     const uint8_t *msg,
                                     uintptr_t msg_len,
                                     const Ec512DoubleScalarBytes *signature);

/**
 * @brief Уничтожить ключ подписи кривой `id-tc26-gost-3410-12-512-paramSetA`.
 *
 * Очищает переданный ключ и обнуляет память.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование ключа после вызова данной функции приводит к неопределённому поведению.
 *
 * @param sk — ненулевой указатель на закрытый ключ
 *
 * \see #Ec2012_512A_SignatureKey
 */
void ec_2012_512a_secret_key_drop(struct Ec2012_512A_SignatureKey *sk);

/**
 * @brief Выработать 256-битный общий ключ используя ключи кривой `id-tc26-gost-3410-12-512-paramSetB`.
 *
 * Выработка общего ключа производится на основе ключа подписи стороны 1 и
 * ключа верификации подписи стороны 2. При условии использования одинакового `ukm`,
 * использование ключа подписи стороны 2 и ключа верификации подписи стороны 1
 * вернёт тот же самый общий ключ.
 *
 * Если в `ukm` передан нулевой указатель, то используется значение по
 * умолчанию равное единице. Если указатель не нулевой, то значение на
 * которое он указывает не должно быть равным нулю по модулю простого
 * числа эллиптической кривой. В случае передачи такого значения
 * возвращается ошибка. Если оба используемых ключа статичны
 * (т.е. используются неэфемерные ключи), то рекомендуется использовать
 * `ukm`, содержащий не менее 128 переменных бит.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * (Для параметра `ukm` нулевое значение допустимо.)
 *
 * @param sk — свой ключ формирования подписи.
 * @param vk — чужой ключ проверки подписи.
 * @param ukm — указатель на пользовательский ключевой материал (опционально).
 * @param out — результирующий общий ключ.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_512B_SignatureKey
 * \see #Ec2012_512B_VerificationKey
 */
intptr_t ec_2012_512b_vko256(const struct Ec2012_512B_SignatureKey *sk,
                             const struct Ec2012_512B_VerificationKey *vk,
                             const Ec512ScalarBytes *ukm,
                             Ec256ScalarBytes *key_dst);

/**
 * @brief Выработать 512-битный общий ключ используя ключи кривой `id-tc26-gost-3410-12-512-paramSetB`.
 *
 * Выработка общего ключа производится на основе ключа подписи стороны 1 и
 * ключа верификации подписи стороны 2. При условии использования одинакового `ukm`,
 * использование ключа подписи стороны 2 и ключа верификации подписи стороны 1
 * вернёт тот же самый общий ключ.
 *
 * Если в `ukm` передан нулевой указатель, то используется значение по
 * умолчанию равное единице. Если указатель не нулевой, то значение на
 * которое он указывает не должно быть равным нулю по модулю простого
 * числа эллиптической кривой. В случае передачи такого значения
 * возвращается ошибка. Если оба используемых ключа статичны
 * (т.е. используются неэфемерные ключи), то рекомендуется использовать
 * `ukm`, содержащий не менее 128 переменных бит.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * (Для параметра `ukm` нулевое значение допустимо.)
 *
 * @param sk — свой ключ формирования подписи.
 * @param vk — чужой ключ проверки подписи.
 * @param ukm — указатель на пользовательский ключевой материал (опционально).
 * @param out — результирующий общий ключ.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_512B_SignatureKey
 * \see #Ec2012_512B_VerificationKey
 */
intptr_t ec_2012_512b_vko512(const struct Ec2012_512B_SignatureKey *sk,
                             const struct Ec2012_512B_VerificationKey *pk,
                             const Ec512ScalarBytes *ukm,
                             Ec512ScalarBytes *key_dst);

/**
 * @brief Десериалировать ключ подписи кривой `id-tc26-gost-3410-12-512-paramSetB`.
 *
 * Десериализует ключ подписи из представления определённого в
 * ГОСТ 34.10-2018 раздел 5.3 во внутреннее представление.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk_bytes — сериализованное представление ключа подписи.
 * @param sk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ подписи для кривой.
 *
 * \see #Ec2012_512B_SignatureKey
 */
intptr_t ec_2012_512b_deserialize_signature_key(const Ec512ScalarBytes *sk_bytes,
                                                struct Ec2012_512B_SignatureKey *sk_dst);

/**
 * @brief Cериалировать ключ подписи кривой `id-tc26-gost-3410-12-512-paramSetB`.
 *
 * Cериализует ключ подписи из внутреннего представления в представление определённое
 * в ГОСТ 34.10-2018 раздел 5.3.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ подписи.
 * @param sk_bytes_dst — указатель на память в которую будет записан сериализованный ключ.
 *
 * \see #Ec2012_512B_SignatureKey
 */
void ec_2012_512b_serialize_signature_key(const struct Ec2012_512B_SignatureKey *sk,
                                          Ec512ScalarBytes *sk_bytes_dst);

/**
 * @brief Получить ключ проверки подписи из ключа формирования подписи.
 *
 * Получает ключ проверки подписи из ключа формирования подписи. Оба ключа принадлежат
 * к кривой `id-tc26-gost-3410-12-512-paramSetB`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param vk_dst — результирующий ключ проверки подписи.
 *
 * \see #Ec2012_512B_SignatureKey
 * \see #Ec2012_512B_VerificationKey
 */
void ec_2012_512b_get_verification_key(const struct Ec2012_512B_SignatureKey *sk,
                                       struct Ec2012_512B_VerificationKey *vk_dst);

/**
 * @brief Десериалировать ключ проверки подписи кривой `id-tc26-gost-3410-12-512-paramSetB`.
 *
 * Десериализует ключ проверки подписи из представления определённого в
 * ГОСТ 34.10-2018 раздел 5.3 во внутреннее
 * представление.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk_bytes — сериализованное представление ключа проверки подписи.
 * @param vk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ проверки подписи для кривой.
 *
 * \see #Ec2012_512B_VerificationKey
 */
intptr_t ec_2012_512b_deserialize_verification_key(const Ec512DoubleScalarBytes *vk_bytes,
                                                   struct Ec2012_512B_VerificationKey *vk_dst);

/**
 * @brief Cериалировать ключ проверки подписи кривой `id-tc26-gost-3410-12-512-paramSetB`.
 *
 * Cериализует ключ проверки подписи из внутреннего представления в представление
 * определённое в ГОСТ 34.10-2018 раздел 5.3.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk_bytes — сериализованное представление ключа проверки подписи.
 * @param vk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ проверки подписи для кривой.
 *
 * \see #Ec2012_512B_VerificationKey
 */
void ec_2012_512b_serialize_verification_key(const struct Ec2012_512B_VerificationKey *vk,
                                             Ec512DoubleScalarBytes *vk_bytes_dst);

/**
 * @brief Подписать хэш-код сообщения используя ключ подписи кривой `id-tc26-gost-3410-12-512-paramSetB`.
 *
 * Формирует электронную подпись для хэш-код сообщения с помощью ключа подписи
 * принадлежащего `id-tc26-gost-3410-12-512-paramSetB`. Согласно ГОСТ 34.10-2018 хэш-код сообщения
 * должен быть вычислен с помощью функции Стрибог определённой в ГОСТ 34.11-2018.
 *
 * Функция возвращает ошибку если переданные `rand_bytes` не позволяют сформировать
 * корректную цифровую подпись. В таком случае данную функцию следует вызвать заново
 * с новым значением `rand_bytes`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param hash — хэш-код сообщения.
 * @param rand_bytes — набор чисел, сгенерированный случайным образом.
 * @param signature — результирующая подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_512B_SignatureKey
 */
intptr_t ec_2012_512b_dsa_sign_hash(const struct Ec2012_512B_SignatureKey *sk,
                                    const Streebog512Result *hash,
                                    const Ec512ScalarBytes *rand_bytes,
                                    Ec512DoubleScalarBytes *signature);

/**
 * @brief Подписать сообщения используя ключ подписи кривой `id-tc26-gost-3410-12-512-paramSetB`.
 *
 * Формирует электронную подпись для сообщения с помощью ключа подписи
 * принадлежащего `id-tc26-gost-3410-12-512-paramSetB`.
 *
 * Функция возвращает ошибку если переданные `rand_bytes` не позволяют сформировать
 * корректную цифровую подпись. В таком случае данную функцию следует вызвать заново
 * с новым значением `rand_bytes`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param msg — указатель на сообщение.
 * @param msg_len — длина сообщения.
 * @param rand_bytes — набор чисел, сгенерированный случайным образом.
 * @param signature_dst — указатель на память в которую будет записана результирующая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_512B_SignatureKey
 */
intptr_t ec_2012_512b_dsa_sign_msg(const struct Ec2012_512B_SignatureKey *sk,
                                   const uint8_t *msg,
                                   uintptr_t msg_len,
                                   const Ec512ScalarBytes *rand_bytes,
                                   Ec512DoubleScalarBytes *signature_dst);

/**
 * @brief Проверить подпись хэш-кода сообщения используя ключ проверки подписи кривой `id-tc26-gost-3410-12-512-paramSetB`.
 *
 * Проверяет электронную подпись хэш-кода сообщения с помощью ключа проверки подписи
 * принадлежащего `id-tc26-gost-3410-12-512-paramSetB`. Согласно ГОСТ 34.10-2018 хэш-код сообщения
 * должен быть вычислен с помощью функции Стрибог определённой в ГОСТ 34.11-2018.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk — ключ проверки подписи.
 * @param hash — хэш-код сообщения.
 * @param signature — проверяемая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибочности цифровой подписи
 *
 * \see #Ec2012_512B_VerificationKey
 */
intptr_t ec_2012_512b_verify_hash_sig(const struct Ec2012_512B_VerificationKey *vk,
                                      const Streebog512Result *hash,
                                      const Ec512DoubleScalarBytes *signature);

/**
 * @brief Проверить подпись сообщения используя ключ проверки подписи кривой `id-tc26-gost-3410-12-512-paramSetB`.
 *
 * Проверяет электронную подпись сообщения с помощью ключа проверки подписи принадлежащего `id-tc26-gost-3410-12-512-paramSetB`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk — ключ проверки подписи.
 * @param msg — указатель на сообщение.
 * @param msg_len — длина сообщения.
 * @param signature — проверяемая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибочности цифровой подписи
 *
 * \see #Ec2012_512B_VerificationKey
 */
intptr_t ec_2012_512b_verify_msg_sig(const struct Ec2012_512B_VerificationKey *vk,
                                     const uint8_t *msg,
                                     uintptr_t msg_len,
                                     const Ec512DoubleScalarBytes *signature);

/**
 * @brief Уничтожить ключ подписи кривой `id-tc26-gost-3410-12-512-paramSetB`.
 *
 * Очищает переданный ключ и обнуляет память.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование ключа после вызова данной функции приводит к неопределённому поведению.
 *
 * @param sk — ненулевой указатель на закрытый ключ
 *
 * \see #Ec2012_512B_SignatureKey
 */
void ec_2012_512b_secret_key_drop(struct Ec2012_512B_SignatureKey *sk);

/**
 * @brief Выработать 256-битный общий ключ используя ключи кривой `id-tc26-gost-3410-12-512-paramSetC`.
 *
 * Выработка общего ключа производится на основе ключа подписи стороны 1 и
 * ключа верификации подписи стороны 2. При условии использования одинакового `ukm`,
 * использование ключа подписи стороны 2 и ключа верификации подписи стороны 1
 * вернёт тот же самый общий ключ.
 *
 * Если в `ukm` передан нулевой указатель, то используется значение по
 * умолчанию равное единице. Если указатель не нулевой, то значение на
 * которое он указывает не должно быть равным нулю по модулю простого
 * числа эллиптической кривой. В случае передачи такого значения
 * возвращается ошибка. Если оба используемых ключа статичны
 * (т.е. используются неэфемерные ключи), то рекомендуется использовать
 * `ukm`, содержащий не менее 128 переменных бит.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * (Для параметра `ukm` нулевое значение допустимо.)
 *
 * @param sk — свой ключ формирования подписи.
 * @param vk — чужой ключ проверки подписи.
 * @param ukm — указатель на пользовательский ключевой материал (опционально).
 * @param out — результирующий общий ключ.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_512C_SignatureKey
 * \see #Ec2012_512C_VerificationKey
 */
intptr_t ec_2012_512c_vko256(const struct Ec2012_512C_SignatureKey *sk,
                             const struct Ec2012_512C_VerificationKey *vk,
                             const Ec512ScalarBytes *ukm,
                             Ec256ScalarBytes *key_dst);

/**
 * @brief Выработать 512-битный общий ключ используя ключи кривой `id-tc26-gost-3410-12-512-paramSetC`.
 *
 * Выработка общего ключа производится на основе ключа подписи стороны 1 и
 * ключа верификации подписи стороны 2. При условии использования одинакового `ukm`,
 * использование ключа подписи стороны 2 и ключа верификации подписи стороны 1
 * вернёт тот же самый общий ключ.
 *
 * Если в `ukm` передан нулевой указатель, то используется значение по
 * умолчанию равное единице. Если указатель не нулевой, то значение на
 * которое он указывает не должно быть равным нулю по модулю простого
 * числа эллиптической кривой. В случае передачи такого значения
 * возвращается ошибка. Если оба используемых ключа статичны
 * (т.е. используются неэфемерные ключи), то рекомендуется использовать
 * `ukm`, содержащий не менее 128 переменных бит.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * (Для параметра `ukm` нулевое значение допустимо.)
 *
 * @param sk — свой ключ формирования подписи.
 * @param vk — чужой ключ проверки подписи.
 * @param ukm — указатель на пользовательский ключевой материал (опционально).
 * @param out — результирующий общий ключ.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_512C_SignatureKey
 * \see #Ec2012_512C_VerificationKey
 */
intptr_t ec_2012_512c_vko512(const struct Ec2012_512C_SignatureKey *sk,
                             const struct Ec2012_512C_VerificationKey *pk,
                             const Ec512ScalarBytes *ukm,
                             Ec512ScalarBytes *key_dst);

/**
 * @brief Десериалировать ключ подписи кривой `id-tc26-gost-3410-12-512-paramSetC`.
 *
 * Десериализует ключ подписи из представления определённого в
 * ГОСТ 34.10-2018 раздел 5.3 во внутреннее представление.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk_bytes — сериализованное представление ключа подписи.
 * @param sk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ подписи для кривой.
 *
 * \see #Ec2012_512C_SignatureKey
 */
intptr_t ec_2012_512c_deserialize_signature_key(const Ec512ScalarBytes *sk_bytes,
                                                struct Ec2012_512C_SignatureKey *sk_dst);

/**
 * @brief Cериалировать ключ подписи кривой `id-tc26-gost-3410-12-512-paramSetC`.
 *
 * Cериализует ключ подписи из внутреннего представления в представление определённое
 * в ГОСТ 34.10-2018 раздел 5.3.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ подписи.
 * @param sk_bytes_dst — указатель на память в которую будет записан сериализованный ключ.
 *
 * \see #Ec2012_512C_SignatureKey
 */
void ec_2012_512c_serialize_signature_key(const struct Ec2012_512C_SignatureKey *sk,
                                          Ec512ScalarBytes *sk_bytes_dst);

/**
 * @brief Получить ключ проверки подписи из ключа формирования подписи.
 *
 * Получает ключ проверки подписи из ключа формирования подписи. Оба ключа принадлежат
 * к кривой `id-tc26-gost-3410-12-512-paramSetC`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param vk_dst — результирующий ключ проверки подписи.
 *
 * \see #Ec2012_512C_SignatureKey
 * \see #Ec2012_512C_VerificationKey
 */
void ec_2012_512c_get_verification_key(const struct Ec2012_512C_SignatureKey *sk,
                                       struct Ec2012_512C_VerificationKey *vk_dst);

/**
 * @brief Десериалировать ключ проверки подписи кривой `id-tc26-gost-3410-12-512-paramSetC`.
 *
 * Десериализует ключ проверки подписи из представления определённого в
 * ГОСТ 34.10-2018 раздел 5.3 во внутреннее
 * представление.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk_bytes — сериализованное представление ключа проверки подписи.
 * @param vk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ проверки подписи для кривой.
 *
 * \see #Ec2012_512C_VerificationKey
 */
intptr_t ec_2012_512c_deserialize_verification_key(const Ec512DoubleScalarBytes *vk_bytes,
                                                   struct Ec2012_512C_VerificationKey *vk_dst);

/**
 * @brief Cериалировать ключ проверки подписи кривой `id-tc26-gost-3410-12-512-paramSetC`.
 *
 * Cериализует ключ проверки подписи из внутреннего представления в представление
 * определённое в ГОСТ 34.10-2018 раздел 5.3.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk_bytes — сериализованное представление ключа проверки подписи.
 * @param vk_dst — указатель на память в которую будет записан десериализованный ключ.
 *
 * @returns 0, в случае успеха, -1, в случае если переданные байты не представляют
 * корректный ключ проверки подписи для кривой.
 *
 * \see #Ec2012_512C_VerificationKey
 */
void ec_2012_512c_serialize_verification_key(const struct Ec2012_512C_VerificationKey *vk,
                                             Ec512DoubleScalarBytes *vk_bytes_dst);

/**
 * @brief Подписать хэш-код сообщения используя ключ подписи кривой `id-tc26-gost-3410-12-512-paramSetC`.
 *
 * Формирует электронную подпись для хэш-код сообщения с помощью ключа подписи
 * принадлежащего `id-tc26-gost-3410-12-512-paramSetC`. Согласно ГОСТ 34.10-2018 хэш-код сообщения
 * должен быть вычислен с помощью функции Стрибог определённой в ГОСТ 34.11-2018.
 *
 * Функция возвращает ошибку если переданные `rand_bytes` не позволяют сформировать
 * корректную цифровую подпись. В таком случае данную функцию следует вызвать заново
 * с новым значением `rand_bytes`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param hash — хэш-код сообщения.
 * @param rand_bytes — набор чисел, сгенерированный случайным образом.
 * @param signature — результирующая подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_512C_SignatureKey
 */
intptr_t ec_2012_512c_dsa_sign_hash(const struct Ec2012_512C_SignatureKey *sk,
                                    const Streebog512Result *hash,
                                    const Ec512ScalarBytes *rand_bytes,
                                    Ec512DoubleScalarBytes *signature);

/**
 * @brief Подписать сообщения используя ключ подписи кривой `id-tc26-gost-3410-12-512-paramSetC`.
 *
 * Формирует электронную подпись для сообщения с помощью ключа подписи
 * принадлежащего `id-tc26-gost-3410-12-512-paramSetC`.
 *
 * Функция возвращает ошибку если переданные `rand_bytes` не позволяют сформировать
 * корректную цифровую подпись. В таком случае данную функцию следует вызвать заново
 * с новым значением `rand_bytes`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param sk — ключ формирования подписи.
 * @param msg — указатель на сообщение.
 * @param msg_len — длина сообщения.
 * @param rand_bytes — набор чисел, сгенерированный случайным образом.
 * @param signature_dst — указатель на память в которую будет записана результирующая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки
 *
 * \see #Ec2012_512C_SignatureKey
 */
intptr_t ec_2012_512c_dsa_sign_msg(const struct Ec2012_512C_SignatureKey *sk,
                                   const uint8_t *msg,
                                   uintptr_t msg_len,
                                   const Ec512ScalarBytes *rand_bytes,
                                   Ec512DoubleScalarBytes *signature_dst);

/**
 * @brief Проверить подпись хэш-кода сообщения используя ключ проверки подписи кривой `id-tc26-gost-3410-12-512-paramSetC`.
 *
 * Проверяет электронную подпись хэш-кода сообщения с помощью ключа проверки подписи
 * принадлежащего `id-tc26-gost-3410-12-512-paramSetC`. Согласно ГОСТ 34.10-2018 хэш-код сообщения
 * должен быть вычислен с помощью функции Стрибог определённой в ГОСТ 34.11-2018.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk — ключ проверки подписи.
 * @param hash — хэш-код сообщения.
 * @param signature — проверяемая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибочности цифровой подписи
 *
 * \see #Ec2012_512C_VerificationKey
 */
intptr_t ec_2012_512c_verify_hash_sig(const struct Ec2012_512C_VerificationKey *vk,
                                      const Streebog512Result *hash,
                                      const Ec512DoubleScalarBytes *signature);

/**
 * @brief Проверить подпись сообщения используя ключ проверки подписи кривой `id-tc26-gost-3410-12-512-paramSetC`.
 *
 * Проверяет электронную подпись сообщения с помощью ключа проверки подписи принадлежащего `id-tc26-gost-3410-12-512-paramSetC`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param vk — ключ проверки подписи.
 * @param msg — указатель на сообщение.
 * @param msg_len — длина сообщения.
 * @param signature — проверяемая электронная подпись.
 *
 * @returns 0, в случае успеха, -1, в случае ошибочности цифровой подписи
 *
 * \see #Ec2012_512C_VerificationKey
 */
intptr_t ec_2012_512c_verify_msg_sig(const struct Ec2012_512C_VerificationKey *vk,
                                     const uint8_t *msg,
                                     uintptr_t msg_len,
                                     const Ec512DoubleScalarBytes *signature);

/**
 * @brief Уничтожить ключ подписи кривой `id-tc26-gost-3410-12-512-paramSetC`.
 *
 * Очищает переданный ключ и обнуляет память.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование ключа после вызова данной функции приводит к неопределённому поведению.
 *
 * @param sk — ненулевой указатель на закрытый ключ
 *
 * \see #Ec2012_512C_SignatureKey
 */
void ec_2012_512c_secret_key_drop(struct Ec2012_512C_SignatureKey *sk);

/**
 * @brief Создать состояние функции хэширования.
 *
 * Создаёт новый состояние функции хэширования.
 *
 * @param dst — ненулевой указатель на память для хранения контекста хещ-функции.
 *
 * \see #Streebog256
 */
void streebog256_new(struct Streebog256 *dst);

/**
 * @brief Хэшировать сообщение.
 *
 * Хэширует массив данных `msg` и копирует результат в массив `hash_dst`.
 *
 * \warning Передача нулевых или некорректных указателей приводит к неопределённому поведению.
 *
 * @param msg — ненулевой указатель на массив байт.
 * @param len — длина сообщения в байтах.
 * @param hash_dst — ненулевой указатель на массив байт для хранения полученного хеш-кода.
 *
 * \see #Streebog256
 */
void streebog256_digest(const uint8_t *msg,
                        uintptr_t len,
                        Streebog256Result *hash_dst);

/**
 * @brief Обновить состояние функции хэширования.
 *
 * Скармливает сообщение функции хэширования, обновляя её состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования.
 * @param msg — ненулевой указатель на массив байт.
 * @param len — длина сообщения в байтах.
 *
 * \see #Streebog256
 */
void streebog256_update(struct Streebog256 *state,
                        const uint8_t *msg,
                        uintptr_t len);

/**
 * @brief Финализировать состояние функции хэширования.
 *
 * Финализирует состояние функции хэширования и копирует результирующий хэш-код в массив `res`.
 * Уничтожает состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования.
 * @param hash_dst — ненулевой указатель на массив байт для хранения полученного хеш-кода.
 *
 * \see #Streebog256
 */
void streebog256_finalize(struct Streebog256 *state,
                          Streebog256Result *hash_dst);

/**
 * @brief Финализировать состояние функции хэширования и заменить его новым.
 *
 * Финализирует состояние функции хэширования и копирует результирующий хэш-код в массив `res`.
 * Заменяет существующий состояние на новый.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования.
 * @param hash_dst — ненулевой указатель на массив байт для хранения полученного хеш-кода.
 *
 * \see #Streebog256
 */
void streebog256_finalize_reset(struct Streebog256 *state,
                                Streebog256Result *hash_dst);

/**
 * @brief Сбросить состояние функции хэширования.
 *
 * Заменяет состояние функции хэширования новым состоянием.
 *
 * @param state — ненулевой указатель на состояние функции хэширования.
 *
 * \see #Streebog256
 */
void streebog256_reset(struct Streebog256 *state);

/**
 * @brief Уничтожить состояние функции хэширования.
 *
 * Уничтожает состояние функции хэширования.
 *
 * \warning Использование указателя на состояние после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования.
 *
 * \see #Streebog256
 */
void streebog256_drop(struct Streebog256 *state);

/**
 * @brief Создать состояние функции хэширования.
 *
 * Создаёт новый состояние функции хэширования.
 *
 * @param dst — ненулевой указатель на память для хранения контекста хещ-функции.
 *
 * \see #Streebog512
 */
void streebog512_new(struct Streebog512 *dst);

/**
 * @brief Хэшировать сообщение.
 *
 * Хэширует массив данных `msg` и копирует результат в массив `hash_dst`.
 *
 * \warning Передача нулевых или некорректных указателей приводит к неопределённому поведению.
 *
 * @param msg — ненулевой указатель на массив байт.
 * @param len — длина сообщения в байтах.
 * @param hash_dst — ненулевой указатель на массив байт для хранения полученного хеш-кода.
 *
 * \see #Streebog512
 */
void streebog512_digest(const uint8_t *msg,
                        uintptr_t len,
                        Streebog512Result *hash_dst);

/**
 * @brief Обновить состояние функции хэширования.
 *
 * Скармливает сообщение функции хэширования, обновляя её состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования.
 * @param msg — ненулевой указатель на массив байт.
 * @param len — длина сообщения в байтах.
 *
 * \see #Streebog512
 */
void streebog512_update(struct Streebog512 *state,
                        const uint8_t *msg,
                        uintptr_t len);

/**
 * @brief Финализировать состояние функции хэширования.
 *
 * Финализирует состояние функции хэширования и копирует результирующий хэш-код в массив `res`.
 * Уничтожает состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования.
 * @param hash_dst — ненулевой указатель на массив байт для хранения полученного хеш-кода.
 *
 * \see #Streebog512
 */
void streebog512_finalize(struct Streebog512 *state,
                          Streebog512Result *hash_dst);

/**
 * @brief Финализировать состояние функции хэширования и заменить его новым.
 *
 * Финализирует состояние функции хэширования и копирует результирующий хэш-код в массив `res`.
 * Заменяет существующий состояние на новый.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования.
 * @param hash_dst — ненулевой указатель на массив байт для хранения полученного хеш-кода.
 *
 * \see #Streebog512
 */
void streebog512_finalize_reset(struct Streebog512 *state,
                                Streebog512Result *hash_dst);

/**
 * @brief Сбросить состояние функции хэширования.
 *
 * Заменяет состояние функции хэширования новым состоянием.
 *
 * @param state — ненулевой указатель на состояние функции хэширования.
 *
 * \see #Streebog512
 */
void streebog512_reset(struct Streebog512 *state);

/**
 * @brief Уничтожить состояние функции хэширования.
 *
 * Уничтожает состояние функции хэширования.
 *
 * \warning Использование указателя на состояние после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние функции хэширования.
 *
 * \see #Streebog512
 */
void streebog512_drop(struct Streebog512 *state);

/**
 * Алгоритм диверсификации `KDF_GOSTR3411_2012_256` определённый
 * в Р 50.1.113-2016.
 *
 * `label` и `seed` являются параметрами задаваемые протоколом,
 * их длины должны быть фиксированы им.
 *
 * @param key_ptr — ненулевой указатель на буфер содержащий ключ.
 * @param key_len — длина ключа в байтах.
 * @param label_ptr — ненулевой указатель на буфер содержащий `label`.
 * @param label_len — длина ключа в байтах.
 * @param seed_ptr — ненулевой указатель на буфер содержащий `seed`.
 * @param seed_len — длина ключа в байтах.
 * @param dst_ptr — ненулевой указатель на массив для записи полученного
 * значения.
 */
void kdf_gostr3411_2012_256(const uint8_t *key_ptr,
                            uintptr_t key_len,
                            const uint8_t *label_ptr,
                            uintptr_t label_len,
                            const uint8_t *seed_ptr,
                            uintptr_t seed_len,
                            Streebog256HmacTag *dst_ptr);

/**
 * Алгоритм диверсификации `KDF_TREE_GOSTR3411_2012_256` определённый
 * в Р 50.1.113-2016.
 *
 * `label` и `seed` являются параметрами задаваемые протоколом,
 * их длины должны быть фиксированы им.
 *
 * @param key_ptr — ненулевой указатель на буфер содержащий ключ.
 * @param key_len — длина ключа в байтах.
 * @param label_ptr — ненулевой указатель на буфер содержащий `label`.
 * @param label_len — длина ключа в байтах.
 * @param seed_ptr — ненулевой указатель на буфер содержащий `seed`.
 * @param seed_len — длина ключа в байтах.
 * @param r — внешний фиксируемый параметр, с возможными значениями 1, 2, 3, 4.
 * @param dst_ptr — ненулевой указатель на буфер для записи полученного
 * значения.
 * @param dst_len — длина вырабатываемого значения.
 *
 * @returns 0, в случае успеха, -1 в случае некорректного значения `r`, либо
 * при превышении `dst_len` максимального значения.
 */
intptr_t kdf_tree_gostr3411_2012_256(const uint8_t *key_ptr,
                                     uintptr_t key_len,
                                     const uint8_t *label_ptr,
                                     uintptr_t label_len,
                                     const uint8_t *seed_ptr,
                                     uintptr_t seed_len,
                                     uint8_t r,
                                     uint8_t *dst_ptr,
                                     uintptr_t dst_len);

/**
 * @brief Выработать ключ на основе пароля согласно Р 50.1.111-2016.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param password — ненулевой указатель на пароль.
 * @param password_len — длина пароля (в байтах).
 * @param salt — ненулевой указатель на случайное значение.
 * @param salt_len — длина случайного значения (в байтах).
 * @param rounds — число итераций алгоритма.
 * @param dst — ненулевой указатель на буфер для хранения результата.
 * @param dst_len — требуемая длина результата (в байтах).
 */
void pbkdf2_hmac_streebog512(const uint8_t *password,
                             uintptr_t password_len,
                             const uint8_t *salt,
                             uintptr_t salt_len,
                             uint32_t rounds,
                             uint8_t *dst,
                             uintptr_t dst_len);

/**
 * @brief Создать состояние режима выработки имитовставки.
 *
 * Создаёт новый состояние режима выработки имитовставки с использованием ключа `key`.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния.
 * режима выработки имитовставки.
 * @param key — ненулевой указатель на ключ инициализации режима.
 *
 * \see #KuznyechikCmac
 */
void kuznyechik_cmac_new(struct KuznyechikCmac *dst,
                         const KuznyechikKey *key);

/**
 * @brief Обновить состояние режиму выработки имитовставки.
 *
 * Скармливает сообщение режиму выработки имитовставки, обновляя его состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param msg — ненулевой указатель на массив байт.
 * @param len — длина сообщения в байтах.
 *
 * \see #KuznyechikCmac
 */
void kuznyechik_cmac_update(struct KuznyechikCmac *state,
                            const uint8_t *msg,
                            uintptr_t len);

/**
 * @brief Финализировать состояние режима выработки имитовставки.
 *
 * Финализирует состояние режима выработки имитовставки и копирует результирующую имитовставку
 * в массив `res`. Уничтожает состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param res — ненулевой указатель на память для хранения имитовставки.
 *
 * \see #KuznyechikCmac
 */
void kuznyechik_cmac_finalize(struct KuznyechikCmac *state,
                              KuznyechikTag *res);

/**
 * @brief Финализировать состояние режима выработки имитовставки и заменить его новым.
 *
 * Финализирует состояние режима выработки имитовставки и копирует результирующую имитовставку
 * в массив `res`. Заменяет существующий состояние на новый.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param res — ненулевой указатель на память для хранения имитовставки.
 *
 * \see #KuznyechikCmac
 */
void kuznyechik_cmac_finalize_reset(struct KuznyechikCmac *state,
                                    KuznyechikTag *res);

/**
 * @brief Проверить целостность данных.
 *
 * Проверяет соответствие данных, скормленных режиму выработки имитовставки, данной имитовставке.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param tag — ненулевой указатель на имитовставку.
 *
 * @returns true, в случае соответствия, false в случае несоответствия.
 *
 * \see #KuznyechikCmac
 */
bool kuznyechik_cmac_verify(struct KuznyechikCmac *state,
                            const KuznyechikTag *tag);

/**
 * @brief Сбросить состояние режима выработки имитовставки.
 *
 * Заменяет состояние режима выработки имитовставки новым состоянием.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 *
 * \see #KuznyechikCmac
 */
void kuznyechik_cmac_reset(struct KuznyechikCmac *state);

/**
 * @brief Уничтожить состояние режима выработки имитовставки.
 *
 * Уничтожает состояние режима выработки имитовставки.
 *
 * \warning Использование указателя на состояние после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 *
 * \see #KuznyechikCmac
 */
void kuznyechik_cmac_drop(struct KuznyechikCmac *state);

/**
 * @brief Создать состояние режима выработки имитовставки.
 *
 * Создаёт контекст блочного режима с использованием указателя на блочный шифр `cipher`.
 * Память с контекстом блочного шифра должна быть действительна на всём протяжении жизни
 * созданного контекста режима выработки имитовставки. Иначе говоря, созданный контекст
 * хранит внутри себя указатель `cipher`.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния
 * режима выработки имитовставки.
 * @param cipher — ненулевой указатель на инициализированное состояние блочного шифра #Kuznyechik.
 *
 * \see #KuznyechikRefCmac
 */
void kuznyechik_ref_cmac_new(struct KuznyechikRefCmac *dst,
                             const struct KuznyechikEnc *cipher);

/**
 * @brief Обновить состояние режиму выработки имитовставки.
 *
 * Скармливает сообщение режиму выработки имитовставки, обновляя его состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param msg — ненулевой указатель на массив байт.
 * @param len — длина сообщения в байтах.
 *
 * \see #KuznyechikRefCmac
 */
void kuznyechik_ref_cmac_update(struct KuznyechikRefCmac *state,
                                const uint8_t *msg,
                                uintptr_t len);

/**
 * @brief Финализировать состояние режима выработки имитовставки.
 *
 * Финализирует состояние режима выработки имитовставки и копирует результирующую имитовставку
 * в массив `res`. Уничтожает состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param res — ненулевой указатель на память для хранения имитовставки.
 *
 * \see #KuznyechikRefCmac
 */
void kuznyechik_ref_cmac_finalize(struct KuznyechikRefCmac *state,
                                  KuznyechikTag *res);

/**
 * @brief Финализировать состояние режима выработки имитовставки и заменить его новым.
 *
 * Финализирует состояние режима выработки имитовставки и копирует результирующую имитовставку
 * в массив `res`. Заменяет существующий состояние на новый.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param res — ненулевой указатель на память для хранения имитовставки.
 *
 * \see #KuznyechikRefCmac
 */
void kuznyechik_ref_cmac_finalize_reset(struct KuznyechikRefCmac *state,
                                        KuznyechikTag *res);

/**
 * @brief Проверить целостность данных.
 *
 * Проверяет соответствие данных, скормленных режиму выработки имитовставки, данной имитовставке.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param tag — ненулевой указатель на имитовставку.
 *
 * @returns true, в случае соответствия, false в случае несоответствия.
 *
 * \see #KuznyechikRefCmac
 */
bool kuznyechik_ref_cmac_verify(struct KuznyechikRefCmac *state,
                                const KuznyechikTag *tag);

/**
 * @brief Сбросить состояние режима выработки имитовставки.
 *
 * Заменяет состояние режима выработки имитовставки новым состоянием.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 *
 * \see #KuznyechikRefCmac
 */
void kuznyechik_ref_cmac_reset(struct KuznyechikRefCmac *state);

/**
 * @brief Уничтожить состояние режима выработки имитовставки.
 *
 * Уничтожает состояние режима выработки имитовставки.
 *
 * \warning Использование указателя на состояние после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 *
 * \see #KuznyechikRefCmac
 */
void kuznyechik_ref_cmac_drop(struct KuznyechikRefCmac *state);

/**
 * @brief Создать состояние режима выработки имитовставки.
 *
 * Создаёт новый состояние режима выработки имитовставки с использованием ключа `key`.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния
 * режима выработки имитовставки.
 * @param key — ненулевой указатель на ключ инициализации режима.
 *
 * \see #MagmaCmac
 */
void magma_cmac_new(struct MagmaCmac *dst,
                    const MagmaKey *key);

/**
 * @brief Обновить состояние режиму выработки имитовставки.
 *
 * Скармливает сообщение режиму выработки имитовставки, обновляя его состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param msg — ненулевой указатель на массив байт.
 * @param len — длина сообщения в байтах.
 *
 * \see #MagmaCmac
 */
void magma_cmac_update(struct MagmaCmac *state,
                       const uint8_t *msg,
                       uintptr_t len);

/**
 * @brief Финализировать состояние режима выработки имитовставки.
 *
 * Финализирует состояние режима выработки имитовставки и копирует результирующую имитовставку
 * в массив `res`. Уничтожает состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param res — ненулевой указатель на память для хранения имитовставки.
 *
 * \see #MagmaCmac
 */
void magma_cmac_finalize(struct MagmaCmac *state,
                         MagmaTag *res);

/**
 * @brief Финализировать состояние режима выработки имитовставки и заменить его новым.
 *
 * Финализирует состояние режима выработки имитовставки и копирует результирующую имитовставку
 * в массив `res`. Заменяет существующий состояние на новый.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param res — ненулевой указатель на память для хранения имитовставки.
 *
 * \see #MagmaCmac
 */
void magma_cmac_finalize_reset(struct MagmaCmac *state,
                               MagmaTag *res);

/**
 * @brief Проверить целостность данных.
 *
 * Проверяет соответствие данных, скормленных режиму выработки имитовставки, данной имитовставке.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param tag — ненулевой указатель на имитовставку.
 *
 * @returns true, в случае соответствия, false в случае несоответствия.
 *
 * \see #MagmaCmac
 */
bool magma_cmac_verify(struct MagmaCmac *state,
                       const MagmaTag *tag);

/**
 * @brief Сбросить состояние режима выработки имитовставки.
 *
 * Заменяет состояние режима выработки имитовставки новым состоянием.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 *
 * \see #MagmaCmac
 */
void magma_cmac_reset(struct MagmaCmac *state);

/**
 * @brief Уничтожить состояние режима выработки имитовставки.
 *
 * Уничтожает состояние режима выработки имитовставки.
 *
 * \warning Использование указателя на состояние после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 *
 * \see #MagmaCmac
 */
void magma_cmac_drop(struct MagmaCmac *state);

/**
 * @brief Создать состояние режима выработки имитовставки на основе ссылки на блочный шифр.
 *
 * Создаёт контекст блочного режима с использованием указателя на блочный шифр `cipher`.
 * Память с контекстом блочного шифра должна быть действительна на всём протяжении жизни
 * созданного контекста режима выработки имитовставки. Иначе говоря, созданный контекст
 * хранит внутри себя указатель `cipher`.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния
 * режима выработки имитовставки.
 * @param cipher — ненулевой указатель на инициализированное состояние блочного шифра #Magma.
 *
 * \see #MagmaRefCmac
 */
void magma_ref_cmac_new(struct MagmaRefCmac *dst,
                        const struct Magma *cipher);

/**
 * @brief Обновить состояние режиму выработки имитовставки.
 *
 * Скармливает сообщение режиму выработки имитовставки, обновляя его состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param msg — ненулевой указатель на массив байт.
 * @param len — длина сообщения в байтах.
 *
 * \see #MagmaRefCmac
 */
void magma_ref_cmac_update(struct MagmaRefCmac *state,
                           const uint8_t *msg,
                           uintptr_t len);

/**
 * @brief Финализировать состояние режима выработки имитовставки.
 *
 * Финализирует состояние режима выработки имитовставки и копирует результирующую имитовставку
 * в массив `res`. Уничтожает состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param res — ненулевой указатель на память для хранения имитовставки.
 *
 * \see #MagmaRefCmac
 */
void magma_ref_cmac_finalize(struct MagmaRefCmac *state,
                             MagmaTag *res);

/**
 * @brief Финализировать состояние режима выработки имитовставки и заменить его новым.
 *
 * Финализирует состояние режима выработки имитовставки и копирует результирующую имитовставку
 * в массив `res`. Заменяет существующий состояние на новый.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param res — ненулевой указатель на память для хранения имитовставки.
 *
 * \see #MagmaRefCmac
 */
void magma_ref_cmac_finalize_reset(struct MagmaRefCmac *state,
                                   MagmaTag *res);

/**
 * @brief Проверить целостность данных.
 *
 * Проверяет соответствие данных, скормленных режиму выработки имитовставки, данной имитовставке.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param tag — ненулевой указатель на имитовставку.
 *
 * @returns true, в случае соответствия, false в случае несоответствия.
 *
 * \see #MagmaRefCmac
 */
bool magma_ref_cmac_verify(struct MagmaRefCmac *state,
                           const MagmaTag *tag);

/**
 * @brief Сбросить состояние режима выработки имитовставки.
 *
 * Заменяет состояние режима выработки имитовставки новым состоянием.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 *
 * \see #MagmaRefCmac
 */
void magma_ref_cmac_reset(struct MagmaRefCmac *state);

/**
 * @brief Уничтожить состояние режима выработки имитовставки.
 *
 * Уничтожает состояние режима выработки имитовставки.
 *
 * \warning Использование указателя на состояние после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 *
 * \see #MagmaRefCmac
 */
void magma_ref_cmac_drop(struct MagmaRefCmac *state);

/**
 * @brief Создать состояние режима выработки имитовставки.
 *
 * Создаёт новый состояние режима выработки имитовставки с использованием
 * переданного ключа.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния.
 * режима выработки имитовставки.
 * @param key_ptr — ненулевой указатель на буфер содержащий ключ.
 * @param key_len — длина ключа в байтах.
 *
 * \see #Streebog256Hmac
 */
void streebog256_hmac_new(struct Streebog256Hmac *dst,
                          const uint8_t *key_ptr,
                          uintptr_t key_len);

/**
 * @brief Обновить состояние режиму выработки имитовставки.
 *
 * Скармливает сообщение режиму выработки имитовставки, обновляя его состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param data_ptr — ненулевой указатель на массив байт.
 * @param data_len — длина сообщения в байтах.
 *
 * \see #Streebog256Hmac
 */
void streebog256_hmac_update(struct Streebog256Hmac *state,
                             const uint8_t *data_ptr,
                             uintptr_t data_len);

/**
 * @brief Финализировать состояние режима выработки имитовставки.
 *
 * Финализирует состояние режима выработки имитовставки и копирует результирующую имитовставку
 * в массив `res`. Уничтожает состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param res — ненулевой указатель на память для хранения имитовставки.
 *
 * \see #Streebog256Hmac
 */
void streebog256_hmac_finalize(struct Streebog256Hmac *state,
                               Streebog256HmacTag *res);

/**
 * @brief Проверить целостность данных.
 *
 * Проверяет соответствие данных, скормленных режиму выработки имитовставки, данной имитовставке.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param tag — ненулевой указатель на имитовставку.
 *
 * @returns true, в случае соответствия, false в случае несоответствия.
 *
 * \see #Streebog256Hmac
 */
bool streebog256_hmac_verify(struct Streebog256Hmac *state,
                             const Streebog256HmacTag *tag);

/**
 * @brief Уничтожить состояние режима выработки имитовставки.
 *
 * Уничтожает состояние режима выработки имитовставки.
 *
 * \warning Использование указателя на состояние после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 *
 * \see #Streebog256Hmac
 */
void streebog256_hmac_drop(struct Streebog256Hmac *state);

/**
 * @brief Создать состояние режима выработки имитовставки.
 *
 * Создаёт новый состояние режима выработки имитовставки с использованием
 * переданного ключа.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния.
 * режима выработки имитовставки.
 * @param key_ptr — ненулевой указатель на буфер содержащий ключ.
 * @param key_len — длина ключа в байтах.
 *
 * \see #Streebog512Hmac
 */
void streebog512_hmac_new(struct Streebog512Hmac *dst,
                          const uint8_t *key_ptr,
                          uintptr_t key_len);

/**
 * @brief Обновить состояние режиму выработки имитовставки.
 *
 * Скармливает сообщение режиму выработки имитовставки, обновляя его состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param data_ptr — ненулевой указатель на массив байт.
 * @param data_len — длина сообщения в байтах.
 *
 * \see #Streebog512Hmac
 */
void streebog512_hmac_update(struct Streebog512Hmac *state,
                             const uint8_t *data_ptr,
                             uintptr_t data_len);

/**
 * @brief Финализировать состояние режима выработки имитовставки.
 *
 * Финализирует состояние режима выработки имитовставки и копирует результирующую имитовставку
 * в массив `res`. Уничтожает состояние.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param res — ненулевой указатель на память для хранения имитовставки.
 *
 * \see #Streebog512Hmac
 */
void streebog512_hmac_finalize(struct Streebog512Hmac *state,
                               Streebog512HmacTag *res);

/**
 * @brief Проверить целостность данных.
 *
 * Проверяет соответствие данных, скормленных режиму выработки имитовставки, данной имитовставке.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на состояние после вызова данной функции приводит к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 * @param tag — ненулевой указатель на имитовставку.
 *
 * @returns true, в случае соответствия, false в случае несоответствия.
 *
 * \see #Streebog512Hmac
 */
bool streebog512_hmac_verify(struct Streebog512Hmac *state,
                             const Streebog512HmacTag *tag);

/**
 * @brief Уничтожить состояние режима выработки имитовставки.
 *
 * Уничтожает состояние режима выработки имитовставки.
 *
 * \warning Использование указателя на состояние после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param state — ненулевой указатель на состояние режима выработки имитовставки.
 *
 * \see #Streebog512Hmac
 */
void streebog512_hmac_drop(struct Streebog512Hmac *state);

/**
 * @brief Зашифровать сообщение с присоединёнными данными согласно Р 1323565.1.026–2019.
 *
 * Зашифровывает сообщение с присоединёнными данными и копирует полученное
 * имитовставки в массив `tag`. Возвращает ошибку в случае некорректного `nonce`.
 *
 * `buf_in` и `buf_out` должны быть либо равны, либо участки памяти
 * `buf_in..buf_in + buf_len` и `buf_out..buf_out + buf_len` не должны
 * пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст блочного шифра.
 * @param nonce — ненулевой указатель на синхропосылку. Старший бит `nonce[0]`
 * должен быть равен нулю.
 * @param aad — присоединённые данные.
 * @param aad_len — длина присоединённых данных в байтах.
 * @param buf_in — буфер содержащий данные подлежащие зашифрованию.
 * @param buf_out — буфер для записи зашифрованных данных.
 * @param buf_len — длина буферов `buf_in` и `buf_out` (в байтах).
 * @param tag — ненулевой указатель на массив байт для хранения имитовставки.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки.
 */
intptr_t kuznyechik_mgm_encrypt(const struct KuznyechikEnc *cipher,
                                const Mgm128Nonce *nonce,
                                const uint8_t *aad,
                                uintptr_t aad_len,
                                const uint8_t *buf_in,
                                uint8_t *buf_out,
                                uintptr_t buf_len,
                                Mgm128Tag *tag);

/**
 * @brief Расшифровать сообщение с присоединёнными данными согласно Р 1323565.1.026–2019.
 *
 * Расшифровывает сообщение с присоединёнными данными после проверки значения
 * имитовставки `tag`. Возвращает ошибку в случае, если имитовставка не соответствует
 * шифротексту.
 *
 * `buf_in` и `buf_out` должны быть либо равны, либо участки памяти
 * `buf_in..buf_in + buf_len` и `buf_out..buf_out + buf_len` не должны
 * пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст блочного шифра.
 * @param nonce — ненулевой указатель на синхропосылку. Старший бит `nonce[0]`
 * должен быть равен нулю.
 * @param aad — присоединённые данные.
 * @param aad_len — длина присоединённых данных в байтах.
 * @param buf_in — буфер содержащий данные подлежащие расшифрованию.
 * @param buf_in — буфер содержащий данные подлежащие расшифрованию.
 * @param buf_out — буфер для записи расшифрованных данных.
 * @param buf_len — длина буферов `buf_in` и `buf_out` (в байтах).
 * @param tag — ненулевой указатель на имитовставку.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки.
 */
intptr_t kuznyechik_mgm_decrypt(const struct KuznyechikEnc *cipher,
                                const Mgm128Nonce *nonce,
                                const uint8_t *aad,
                                uintptr_t aad_len,
                                const uint8_t *buf_in,
                                uint8_t *buf_out,
                                uintptr_t buf_len,
                                const Mgm128Tag *tag);

/**
 * @brief Перешифровать сообщение с присоединёнными данными согласно Р 1323565.1.026–2019.
 *
 * Перешифровывает сообщение с присоединёнными данными и копирует соответствующую
 * имитовставку в массив `tag`. Возвращает ошибку в случае если имитовставка
 * не соответствует входному шифротексту и в случае некорректных `nonce`.
 *
 * `buf_in` и `buf_out` должны быть либо равны, либо участки памяти
 * `buf_in..buf_in + buf_len` и `buf_out..buf_out + buf_len` не должны
 * пересекаться.
 *
 * Старший бит обоих `nonce` должен быть равен нулю.
 *
 * При расшифровании проверяется значение имитовставки `tag`, в случае успешного
 * перешифрования новое значение имитовставки записывается в `tag`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param old_cipher — ненулевой указатель на контекст блочного шифра для расшифрования.
 * @param new_cipher — ненулевой указатель на контекст блочного шифра для зашифрования.
 * @param old_nonce — ненулевой указатель на синхропосылку для расшифрования.
 * @param new_nonce — ненулевой указатель на синхропосылку для зашифрования.
 * @param aad — присоединённые данные.
 * @param aad_len — длина присоединённых данных в байтах.
 * @param buf_in — буфер содержащий данные подлежащие перешифрованию.
 * @param buf_out — буфер для записи перешифрованных данных.
 * @param buf_len — длина буферов `buf_in` и `buf_out` (в байтах).
 * @param old_tag — ненулевой указатель на массив байт для имитовставки.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки.
 */
intptr_t kuznyechik_mgm_reencrypt(const struct KuznyechikEnc *old_cipher,
                                  const struct KuznyechikEnc *new_cipher,
                                  const Mgm128Nonce *old_nonce,
                                  const Mgm128Nonce *new_nonce,
                                  const uint8_t *aad,
                                  uintptr_t aad_len,
                                  const uint8_t *buf_in,
                                  uint8_t *buf_out,
                                  uintptr_t buf_len,
                                  Mgm128Tag *tag);

/**
 * @brief Зашифровать сообщение с присоединёнными данными согласно Р 1323565.1.026–2019.
 *
 * Зашифровывает сообщение с присоединёнными данными и копирует полученное
 * имитовставки в массив `tag`. Возвращает ошибку в случае некорректного `nonce`.
 *
 * `buf_in` и `buf_out` должны быть либо равны, либо участки памяти
 * `buf_in..buf_in + buf_len` и `buf_out..buf_out + buf_len` не должны
 * пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст блочного шифра.
 * @param nonce — ненулевой указатель на синхропосылку. Старший бит `nonce[0]`
 * должен быть равен нулю.
 * @param aad — присоединённые данные.
 * @param aad_len — длина присоединённых данных в байтах.
 * @param buf_in — буфер содержащий данные подлежащие зашифрованию.
 * @param buf_out — буфер для записи зашифрованных данных.
 * @param buf_len — длина буферов `buf_in` и `buf_out` (в байтах).
 * @param tag — ненулевой указатель на массив байт для хранения имитовставки.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки.
 */
intptr_t magma_mgm_encrypt(const struct Magma *cipher,
                           const Mgm64Nonce *nonce,
                           const uint8_t *aad,
                           uintptr_t aad_len,
                           const uint8_t *buf_in,
                           uint8_t *buf_out,
                           uintptr_t buf_len,
                           Mgm64Tag *tag);

/**
 * @brief Расшифровать сообщение с присоединёнными данными согласно Р 1323565.1.026–2019.
 *
 * Расшифровывает сообщение с присоединёнными данными после проверки значения
 * имитовставки `tag`. Возвращает ошибку в случае, если имитовставка не соответствует
 * шифротексту.
 *
 * `buf_in` и `buf_out` должны быть либо равны, либо участки памяти
 * `buf_in..buf_in + buf_len` и `buf_out..buf_out + buf_len` не должны
 * пересекаться.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param cipher — ненулевой указатель на контекст блочного шифра.
 * @param nonce — ненулевой указатель на синхропосылку. Старший бит `nonce[0]`
 * должен быть равен нулю.
 * @param aad — присоединённые данные.
 * @param aad_len — длина присоединённых данных в байтах.
 * @param buf_in — буфер содержащий данные подлежащие расшифрованию.
 * @param buf_out — буфер для записи расшифрованных данных.
 * @param buf_len — длина буферов `buf_in` и `buf_out` (в байтах).
 * @param tag — ненулевой указатель на имитовставку.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки.
 */
intptr_t magma_mgm_decrypt(const struct Magma *cipher,
                           const Mgm64Nonce *nonce,
                           const uint8_t *aad,
                           uintptr_t aad_len,
                           const uint8_t *buf_in,
                           uint8_t *buf_out,
                           uintptr_t buf_len,
                           const Mgm64Tag *tag);

/**
 * @brief Перешифровать сообщение с присоединёнными данными согласно Р 1323565.1.026–2019.
 *
 * Перешифровывает сообщение с присоединёнными данными и копирует соответствующую
 * имитовставку в массив `tag`. Возвращает ошибку в случае если имитовставка
 * не соответствует входному шифротексту и в случае некорректных `nonce`.
 *
 * `buf_in` и `buf_out` должны быть либо равны, либо участки памяти
 * `buf_in..buf_in + buf_len` и `buf_out..buf_out + buf_len` не должны
 * пересекаться.
 *
 * Старший бит обоих `nonce` должен быть равен нулю.
 *
 * При расшифровании проверяется значение имитовставки `tag`, в случае успешного
 * перешифрования новое значение имитовставки записывается в `tag`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param old_cipher — ненулевой указатель на контекст блочного шифра для расшифрования.
 * @param new_cipher — ненулевой указатель на контекст блочного шифра для зашифрования.
 * @param old_nonce — ненулевой указатель на синхропосылку для расшифрования.
 * @param new_nonce — ненулевой указатель на синхропосылку для зашифрования.
 * @param aad — присоединённые данные.
 * @param aad_len — длина присоединённых данных в байтах.
 * @param buf_in — буфер содержащий данные подлежащие перешифрованию.
 * @param buf_out — буфер для записи перешифрованных данных.
 * @param buf_len — длина буферов `buf_in` и `buf_out` (в байтах).
 * @param old_tag — ненулевой указатель на массив байт для имитовставки.
 *
 * @returns 0, в случае успеха, -1, в случае ошибки.
 */
intptr_t magma_mgm_reencrypt(const struct Magma *old_cipher,
                             const struct Magma *new_cipher,
                             const Mgm64Nonce *old_nonce,
                             const Mgm64Nonce *new_nonce,
                             const uint8_t *aad,
                             uintptr_t aad_len,
                             const uint8_t *buf_in,
                             uint8_t *buf_out,
                             uintptr_t buf_len,
                             Mgm64Tag *tag);

/**
 * @brief Инициализировать состояние блочного режима.
 *
 * Создаёт контекст блочного шифра в режиме простой замены с зацеплением с использованием
 * ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param key — ненулевой указатель на ключ.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #KuznyechikCbcDec
 */
void kuznyechik_cbc_dec_new(struct KuznyechikCbcDec *dst,
                            const KuznyechikKey *key,
                            const KuznyechikNonce *nonce);

/**
 * @brief Расшифровать массив блоков.
 *
 * Расшифровывает блоки данных с помощью заданного контекста #KuznyechikCbcDec.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks_in — ненулевой указатель на буфер содержащий входные блоки.
 * @param blocks_out — ненулевой указатель на буфер для записи полученных блоков.
 * @param blocks_len — длина буферов blocks_in и blocks_out (в блоках).
 *
 * \see #KuznyechikCbcDec
 */
void kuznyechik_cbc_decrypt_blocks(struct KuznyechikCbcDec *mode,
                                   const KuznyechikBlock *blocks_in,
                                   KuznyechikBlock *blocks_out,
                                   uintptr_t blocks_len);

/**
 * @brief Расшифровать сообщение и убрать дополнение.
 *
 * Расшифровывает сообщение с помощью заданного контекста #KuznyechikCbcDec. Уничтожает
 * контекст шифра.
 *
 * Возвращает ошибку в случае, когда длина буфера не делится на #MAGMA_BLOCK_SIZE или
 * расшифрованное сообщение имеет некорректное дополнение.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * @returns Длину результата, в случае успеха или -1, в случае ошибки.
 *
 * \see #KuznyechikCbcDec
 */
intptr_t kuznyechik_cbc_decrypt_unpad(struct KuznyechikCbcDec *mode,
                                      const uint8_t *buf_in,
                                      uint8_t *buf_out,
                                      uintptr_t buf_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_cbc_dec_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #KuznyechikCbcDec
 */
void kuznyechik_cbc_dec_drop(struct KuznyechikCbcDec *mode);

/**
 * @brief Инициализировать состояние блочного режима.
 *
 * Создаёт контекст блочного шифра в режиме простой замены с зацеплением с использованием
 * ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param key — ненулевой указатель на ключ.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #KuznyechikCbcEnc
 */
void kuznyechik_cbc_enc_new(struct KuznyechikCbcEnc *dst,
                            const KuznyechikKey *key,
                            const KuznyechikNonce *nonce);

/**
 * @brief Зашифровать массив блоков.
 *
 * Шифрует блоки данных с помощью заданного контекста #KuznyechikCbcEnc.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks — ненулевой указатель на массив блоков.
 * @param blocks_len — длина массива блоков (в блоках).
 *
 * \see #KuznyechikCbcEnc
 */
void kuznyechik_cbc_encrypt_blocks(struct KuznyechikCbcEnc *mode,
                                   const KuznyechikBlock *blocks_in,
                                   KuznyechikBlock *blocks_out,
                                   uintptr_t blocks_len);

/**
 * @brief Дополнить и зашифровать сообщение.
 *
 * Зашифровывает сообщение с помощью заданного контекста #KuznyechikCbcEnc. Уничтожает
 * контекст шифра.
 *
 * Байты `0..pos` буфера `buf` используются как сообщение, когда же байты `pos..buf_len`
 * используются как зарезервированное место для дополнения.
 *
 * Возвращает ошибку в случае, когда место для дополнения недостаточно велико.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_in_len — длина буфера `buf_in` (в байтах).
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_out_len — длина буфера `buf_out` (в байтах).
 *
 * @returns Длину шифротекста, в случае успеха или -1, в случае ошибки.
 *
 * \see #KuznyechikCbcEnc
 */
intptr_t kuznyechik_cbc_encrypt_pad(struct KuznyechikCbcEnc *mode,
                                    const uint8_t *buf_in,
                                    uintptr_t buf_in_len,
                                    uint8_t *buf_out,
                                    uintptr_t buf_out_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_cbc_enc_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #KuznyechikCbcEnc
 */
void kuznyechik_cbc_enc_drop(struct KuznyechikCbcEnc *mode);

/**
 * @brief Инициализировать состояние блочного режима.
 *
 * Создаёт контекст блочного шифра в режиме простой замены с зацеплением с использованием
 * ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param key — ненулевой указатель на ключ.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #MagmaCbcDec
 */
void magma_cbc_dec_new(struct MagmaCbcDec *dst,
                       const MagmaKey *key,
                       const MagmaNonce *nonce);

/**
 * @brief Расшифровать массив блоков.
 *
 * Расшифровывает блоки данных с помощью заданного контекста #MagmaCbcDec.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks_in — ненулевой указатель на буфер содержащий входные блоки.
 * @param blocks_out — ненулевой указатель на буфер для записи полученных блоков.
 * @param blocks_len — длина буферов blocks_in и blocks_out (в блоках).
 *
 * \see #MagmaCbcDec
 */
void magma_cbc_decrypt_blocks(struct MagmaCbcDec *mode,
                              const MagmaBlock *blocks_in,
                              MagmaBlock *blocks_out,
                              uintptr_t blocks_len);

/**
 * @brief Расшифровать сообщение и убрать дополнение.
 *
 * Расшифровывает сообщение с помощью заданного контекста #MagmaCbcDec. Уничтожает
 * контекст шифра.
 *
 * Возвращает ошибку в случае, когда длина буфера не делится на #MAGMA_BLOCK_SIZE или
 * расшифрованное сообщение имеет некорректное дополнение.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * @returns Длину результата, в случае успеха или -1, в случае ошибки.
 *
 * \see #MagmaCbcDec
 */
intptr_t magma_cbc_decrypt_unpad(struct MagmaCbcDec *mode,
                                 const uint8_t *buf_in,
                                 uint8_t *buf_out,
                                 uintptr_t buf_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_cbc_dec_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #MagmaCbcDec
 */
void magma_cbc_dec_drop(struct MagmaCbcDec *mode);

/**
 * @brief Инициализировать состояние блочного режима.
 *
 * Создаёт контекст блочного шифра в режиме простой замены с зацеплением с использованием
 * ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param key — ненулевой указатель на ключ.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #MagmaCbcEnc
 */
void magma_cbc_enc_new(struct MagmaCbcEnc *dst,
                       const MagmaKey *key,
                       const MagmaNonce *nonce);

/**
 * @brief Зашифровать массив блоков.
 *
 * Шифрует блоки данных с помощью заданного контекста #MagmaCbcEnc.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks — ненулевой указатель на массив блоков.
 * @param blocks_len — длина массива блоков (в блоках).
 *
 * \see #MagmaCbcEnc
 */
void magma_cbc_encrypt_blocks(struct MagmaCbcEnc *mode,
                              const MagmaBlock *blocks_in,
                              MagmaBlock *blocks_out,
                              uintptr_t blocks_len);

/**
 * @brief Дополнить и зашифровать сообщение.
 *
 * Дополняет и зашифровывает сообщение с помощью заданного контекста #MagmaCbcEnc.
 * Уничтожает контекст шифра.
 *
 * Возвращает ошибку в случае, когда размер выходного буфера недостаточно большой.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_in_len — длина буфера `buf_in` (в байтах).
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_out_len — длина буфера `buf_out` (в байтах).
 *
 * @returns Длину шифротекста, в случае успеха или -1, в случае ошибки.
 *
 * \see #MagmaCbcEnc
 */
intptr_t magma_cbc_encrypt_pad(struct MagmaCbcEnc *mode,
                               const uint8_t *buf_in,
                               uintptr_t buf_in_len,
                               uint8_t *buf_out,
                               uintptr_t buf_out_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_cbc_enc_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #MagmaCbcEnc
 */
void magma_cbc_enc_drop(struct MagmaCbcEnc *mode);

/**
 * @brief Инициализировать состояние блочного режима на основе ссылки на блочный шифр.
 *
 * Создаёт контекст блочного режима с использованием указателя на блочный шифр `cipher`
 * и синхропосылки `nonce`. Память с контекстом блочного шифра должна быть действительна
 * на всём протяжении жизни созданного контекста блочного режима. Иначе говоря, созданный
 * контекст хранит внутри себя указатель `cipher`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param cipher — ненулевой указатель на инициализированное состояние блочного шифра #Kuznyechik.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #KuznyechikRefCbcDec
 */
void kuznyechik_ref_cbc_dec_new(struct KuznyechikRefCbcDec *dst,
                                const struct KuznyechikDec *cipher,
                                const KuznyechikNonce *nonce);

/**
 * @brief Расшифровать массив блоков.
 *
 * Расшифровывает блоки данных с помощью заданного контекста #KuznyechikRefCbcDec.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks_in — ненулевой указатель на буфер содержащий входные блоки.
 * @param blocks_out — ненулевой указатель на буфер для записи полученных блоков.
 * @param blocks_len — длина буферов blocks_in и blocks_out (в блоках).
 *
 * \see #KuznyechikRefCbcDec
 */
void kuznyechik_ref_cbc_decrypt_blocks(struct KuznyechikRefCbcDec *mode,
                                       const KuznyechikBlock *blocks_in,
                                       KuznyechikBlock *blocks_out,
                                       uintptr_t blocks_len);

/**
 * @brief Расшифровать сообщение и убрать дополнение.
 *
 * Расшифровывает сообщение с помощью заданного контекста #KuznyechikRefCbcDec. Уничтожает
 * контекст шифра.
 *
 * Возвращает ошибку в случае, когда длина буфера не делится на #MAGMA_BLOCK_SIZE или
 * расшифрованное сообщение имеет некорректное дополнение.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * @returns Длину результата, в случае успеха или -1, в случае ошибки.
 *
 * \see #KuznyechikRefCbcDec
 */
intptr_t kuznyechik_ref_cbc_decrypt_unpad(struct KuznyechikRefCbcDec *mode,
                                          const uint8_t *buf_in,
                                          uint8_t *buf_out,
                                          uintptr_t buf_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_ref_cbc_dec_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #KuznyechikRefCbcDec
 */
void kuznyechik_ref_cbc_dec_drop(struct KuznyechikRefCbcDec *mode);

/**
 * @brief Инициализировать состояние блочного режима на основе ссылки на блочный шифр.
 *
 * Создаёт контекст блочного режима с использованием указателя на блочный шифр `cipher`
 * и синхропосылки `nonce`. Память с контекстом блочного шифра должна быть действительна
 * на всём протяжении жизни созданного контекста блочного режима. Иначе говоря, созданный
 * контекст хранит внутри себя указатель `cipher`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param cipher — ненулевой указатель на инициализированное состояние блочного шифра #Kuznyechik.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #KuznyechikRefCbcEnc
 */
void kuznyechik_ref_cbc_enc_new(struct KuznyechikRefCbcEnc *dst,
                                const struct KuznyechikEnc *cipher,
                                const KuznyechikNonce *nonce);

/**
 * @brief Зашифровать массив блоков.
 *
 * Шифрует блоки данных с помощью заданного контекста #KuznyechikRefCbcEnc.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks — ненулевой указатель на массив блоков.
 * @param blocks_len — длина массива блоков (в блоках).
 *
 * \see #KuznyechikRefCbcEnc
 */
void kuznyechik_ref_cbc_encrypt_blocks(struct KuznyechikRefCbcEnc *mode,
                                       const KuznyechikBlock *blocks_in,
                                       KuznyechikBlock *blocks_out,
                                       uintptr_t blocks_len);

/**
 * @brief Дополнить и зашифровать сообщение.
 *
 * Зашифровывает сообщение с помощью заданного контекста #KuznyechikRefCbcEnc. Уничтожает
 * контекст шифра.
 *
 * Байты `0..pos` буфера `buf` используются как сообщение, когда же байты `pos..buf_len`
 * используются как зарезервированное место для дополнения.
 *
 * Возвращает ошибку в случае, когда место для дополнения недостаточно велико.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_in_len — длина буфера `buf_in` (в байтах).
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_out_len — длина буфера `buf_out` (в байтах).
 *
 * @returns Длину шифротекста, в случае успеха или -1, в случае ошибки.
 *
 * \see #KuznyechikRefCbcEnc
 */
intptr_t kuznyechik_ref_cbc_encrypt_pad(struct KuznyechikRefCbcEnc *mode,
                                        const uint8_t *buf_in,
                                        uintptr_t buf_in_len,
                                        uint8_t *buf_out,
                                        uintptr_t buf_out_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_ref_cbc_enc_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #KuznyechikRefCbcEnc
 */
void kuznyechik_ref_cbc_enc_drop(struct KuznyechikRefCbcEnc *mode);

/**
 * @brief Инициализировать состояние блочного режима на основе ссылки на блочный шифр.
 *
 * Создаёт контекст блочного режима с использованием указателя на блочный шифр `cipher`
 * и синхропосылки `nonce`. Память с контекстом блочного шифра должна быть действительна
 * на всём протяжении жизни созданного контекста блочного режима. Иначе говоря, созданный
 * контекст хранит внутри себя указатель `cipher`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param cipher — ненулевой указатель на инициализированное состояние блочного шифра #Magma.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #MagmaRefCbcDec
 */
void magma_ref_cbc_dec_new(struct MagmaRefCbcDec *dst,
                           const struct Magma *cipher,
                           const MagmaNonce *nonce);

/**
 * @brief Расшифровать массив блоков.
 *
 * Расшифровывает блоки данных с помощью заданного контекста #MagmaRefCbcDec.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks_in — ненулевой указатель на буфер содержащий входные блоки.
 * @param blocks_out — ненулевой указатель на буфер для записи полученных блоков.
 * @param blocks_len — длина буферов blocks_in и blocks_out (в блоках).
 *
 * \see #MagmaRefCbcDec
 */
void magma_ref_cbc_decrypt_blocks(struct MagmaRefCbcDec *mode,
                                  const MagmaBlock *blocks_in,
                                  MagmaBlock *blocks_out,
                                  uintptr_t blocks_len);

/**
 * @brief Расшифровать сообщение и убрать дополнение.
 *
 * Расшифровывает сообщение с помощью заданного контекста #MagmaRefCbcDec. Уничтожает
 * контекст шифра.
 *
 * Возвращает ошибку в случае, когда длина буфера не делится на #MAGMA_BLOCK_SIZE или
 * расшифрованное сообщение имеет некорректное дополнение.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * @returns Длину результата, в случае успеха или -1, в случае ошибки.
 *
 * \see #MagmaRefCbcDec
 */
intptr_t magma_ref_cbc_decrypt_unpad(struct MagmaRefCbcDec *mode,
                                     const uint8_t *buf_in,
                                     uint8_t *buf_out,
                                     uintptr_t buf_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_ref_cbc_dec_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #MagmaRefCbcDec
 */
void magma_ref_cbc_dec_drop(struct MagmaRefCbcDec *mode);

/**
 * @brief Инициализировать состояние блочного режима на основе ссылки на блочный шифр.
 *
 * Создаёт контекст блочного режима с использованием указателя на блочный шифр `cipher`
 * и синхропосылки `nonce`. Память с контекстом блочного шифра должна быть действительна
 * на всём протяжении жизни созданного контекста блочного режима. Иначе говоря, созданный
 * контекст хранит внутри себя указатель `cipher`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param cipher — ненулевой указатель на инициализированное состояние блочного шифра #Magma.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #MagmaRefCbcEnc
 */
void magma_ref_cbc_enc_new(struct MagmaRefCbcEnc *dst,
                           const struct Magma *cipher,
                           const MagmaNonce *nonce);

/**
 * @brief Зашифровать массив блоков.
 *
 * Шифрует блоки данных с помощью заданного контекста #MagmaRefCbcEnc.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks — ненулевой указатель на массив блоков.
 * @param blocks_len — длина массива блоков (в блоках).
 *
 * \see #MagmaRefCbcEnc
 */
void magma_ref_cbc_encrypt_blocks(struct MagmaRefCbcEnc *mode,
                                  const MagmaBlock *blocks_in,
                                  MagmaBlock *blocks_out,
                                  uintptr_t blocks_len);

/**
 * @brief Дополнить и зашифровать сообщение.
 *
 * Зашифровывает сообщение с помощью заданного контекста #MagmaRefCbcEnc. Уничтожает
 * контекст шифра.
 *
 * Байты `0..pos` буфера `buf` используются как сообщение, когда же байты `pos..buf_len`
 * используются как зарезервированное место для дополнения.
 *
 * Возвращает ошибку в случае, когда место для дополнения недостаточно велико.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_in_len — длина буфера `buf_in` (в байтах).
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_out_len — длина буфера `buf_out` (в байтах).
 *
 * @returns Длину шифротекста, в случае успеха или -1, в случае ошибки.
 *
 * \see #MagmaRefCbcEnc
 */
intptr_t magma_ref_cbc_encrypt_pad(struct MagmaRefCbcEnc *mode,
                                   const uint8_t *buf_in,
                                   uintptr_t buf_in_len,
                                   uint8_t *buf_out,
                                   uintptr_t buf_out_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_ref_cbc_enc_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #MagmaRefCbcEnc
 */
void magma_ref_cbc_enc_drop(struct MagmaRefCbcEnc *mode);

/**
 * @brief Инициализировать состояние блочного режима.
 *
 * Создаёт контекст блочного шифра в режиме простой замены с зацеплением с использованием
 * ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param key — ненулевой указатель на ключ.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #KuznyechikCfbDec
 */
void kuznyechik_cfb_dec_new(struct KuznyechikCfbDec *dst,
                            const KuznyechikKey *key,
                            const KuznyechikNonce *nonce);

/**
 * @brief Расшифровать массив блоков.
 *
 * Расшифровывает блоки данных с помощью заданного контекста #KuznyechikCfbDec.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks_in — ненулевой указатель на буфер содержащий входные блоки.
 * @param blocks_out — ненулевой указатель на буфер для записи полученных блоков.
 * @param blocks_len — длина буферов blocks_in и blocks_out (в блоках).
 *
 * \see #KuznyechikCfbDec
 */
void kuznyechik_cfb_decrypt_blocks(struct KuznyechikCfbDec *mode,
                                   const KuznyechikBlock *blocks_in,
                                   KuznyechikBlock *blocks_out,
                                   uintptr_t blocks_len);

/**
 * @brief Расшифровать сообщение.
 *
 * Зашифровывает сообщение с помощью заданного контекста #KuznyechikCfbDec.
 *
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * \see #KuznyechikCfbDec
 */
void kuznyechik_cfb_decrypt(struct KuznyechikCfbDec *mode,
                            const uint8_t *buf_in,
                            uint8_t *buf_out,
                            uintptr_t buf_len);

/**
 * @brief Расшифровать сообщение и убрать дополнение.
 *
 * Расшифровывает сообщение с помощью заданного контекста #KuznyechikCfbDec. Уничтожает
 * контекст шифра.
 *
 * Возвращает ошибку в случае, когда длина буфера не делится на #MAGMA_BLOCK_SIZE или
 * расшифрованное сообщение имеет некорректное дополнение.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * @returns Длину результата, в случае успеха или -1, в случае ошибки.
 *
 * \see #KuznyechikCfbDec
 */
intptr_t kuznyechik_cfb_decrypt_unpad(struct KuznyechikCfbDec *mode,
                                      const uint8_t *buf_in,
                                      uint8_t *buf_out,
                                      uintptr_t buf_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_cfb_dec_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #KuznyechikCfbDec
 */
void kuznyechik_cfb_dec_drop(struct KuznyechikCfbDec *mode);

/**
 * @brief Инициализировать состояние блочного режима.
 *
 * Создаёт контекст блочного шифра в режиме простой замены с зацеплением с использованием
 * ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param key — ненулевой указатель на ключ.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #KuznyechikCfbEnc
 */
void kuznyechik_cfb_enc_new(struct KuznyechikCfbEnc *dst,
                            const KuznyechikKey *key,
                            const KuznyechikNonce *nonce);

/**
 * @brief Зашифровать массив блоков.
 *
 * Шифрует блоки данных с помощью заданного контекста #KuznyechikCfbEnc.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks_in — ненулевой указатель на буфер содержащий входные блоки.
 * @param blocks_out — ненулевой указатель на буфер для записи полученных блоков.
 * @param blocks_len — длина буферов blocks_in и blocks_out (в блоках).
 *
 * \see #KuznyechikCfbEnc
 */
void kuznyechik_cfb_encrypt_blocks(struct KuznyechikCfbEnc *mode,
                                   const KuznyechikBlock *blocks_in,
                                   KuznyechikBlock *blocks_out,
                                   uintptr_t blocks_len);

/**
 * @brief Зашифровать сообщение.
 *
 * Зашифровывает сообщение с помощью заданного контекста #KuznyechikCfbEnc.
 *
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * \see #KuznyechikCfbEnc
 */
void kuznyechik_cfb_encrypt(struct KuznyechikCfbEnc *mode,
                            const uint8_t *buf_in,
                            uint8_t *buf_out,
                            uintptr_t buf_len);

/**
 * @brief Дополнить и зашифровать сообщение.
 *
 * Зашифровывает сообщение с помощью заданного контекста #KuznyechikCfbEnc. Уничтожает
 * контекст шифра.
 *
 * Байты `0..pos` буфера `buf` используются как сообщение, когда же байты `pos..buf_len`
 * используются как зарезервированное место для дополнения.
 *
 * Возвращает ошибку в случае, когда место для дополнения недостаточно велико.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_in_len — длина буфера `buf_in` (в байтах).
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_out_len — длина буфера `buf_out` (в байтах).
 *
 * @returns Длину шифротекста, в случае успеха или -1, в случае ошибки.
 *
 * \see #KuznyechikCfbEnc
 */
intptr_t kuznyechik_cfb_encrypt_pad(struct KuznyechikCfbEnc *mode,
                                    const uint8_t *buf_in,
                                    uintptr_t buf_in_len,
                                    uint8_t *buf_out,
                                    uintptr_t buf_out_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_cfb_enc_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #KuznyechikCfbEnc
 */
void kuznyechik_cfb_enc_drop(struct KuznyechikCfbEnc *mode);

/**
 * @brief Инициализировать состояние блочного режима.
 *
 * Создаёт контекст блочного шифра в режиме простой замены с зацеплением с использованием
 * ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param key — ненулевой указатель на ключ.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #MagmaCfbDec
 */
void magma_cfb_dec_new(struct MagmaCfbDec *dst,
                       const MagmaKey *key,
                       const MagmaNonce *nonce);

/**
 * @brief Расшифровать массив блоков.
 *
 * Расшифровывает блоки данных с помощью заданного контекста #MagmaCfbDec.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks_in — ненулевой указатель на буфер содержащий входные блоки.
 * @param blocks_out — ненулевой указатель на буфер для записи полученных блоков.
 * @param blocks_len — длина буферов blocks_in и blocks_out (в блоках).
 *
 * \see #MagmaCfbDec
 */
void magma_cfb_decrypt_blocks(struct MagmaCfbDec *mode,
                              const MagmaBlock *blocks_in,
                              MagmaBlock *blocks_out,
                              uintptr_t blocks_len);

/**
 * @brief Расшифровать сообщение.
 *
 * Зашифровывает сообщение с помощью заданного контекста #MagmaCfbDec.
 *
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * \see #MagmaCfbDec
 */
void magma_cfb_decrypt(struct MagmaCfbDec *mode,
                       const uint8_t *buf_in,
                       uint8_t *buf_out,
                       uintptr_t buf_len);

/**
 * @brief Расшифровать сообщение и убрать дополнение.
 *
 * Расшифровывает сообщение с помощью заданного контекста #MagmaCfbDec. Уничтожает
 * контекст шифра.
 *
 * Возвращает ошибку в случае, когда длина буфера не делится на #MAGMA_BLOCK_SIZE или
 * расшифрованное сообщение имеет некорректное дополнение.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * @returns Длину результата, в случае успеха или -1, в случае ошибки.
 *
 * \see #MagmaCfbDec
 */
intptr_t magma_cfb_decrypt_unpad(struct MagmaCfbDec *mode,
                                 const uint8_t *buf_in,
                                 uint8_t *buf_out,
                                 uintptr_t buf_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_cfb_dec_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #MagmaCfbDec
 */
void magma_cfb_dec_drop(struct MagmaCfbDec *mode);

/**
 * @brief Инициализировать состояние блочного режима.
 *
 * Создаёт контекст блочного шифра в режиме простой замены с зацеплением с использованием
 * ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param key — ненулевой указатель на ключ.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #MagmaCfbEnc
 */
void magma_cfb_enc_new(struct MagmaCfbEnc *dst,
                       const MagmaKey *key,
                       const MagmaNonce *nonce);

/**
 * @brief Зашифровать массив блоков.
 *
 * Шифрует блоки данных с помощью заданного контекста #MagmaCfbEnc.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks_in — ненулевой указатель на буфер содержащий входные блоки.
 * @param blocks_out — ненулевой указатель на буфер для записи полученных блоков.
 * @param blocks_len — длина буферов blocks_in и blocks_out (в блоках).
 *
 * \see #MagmaCfbEnc
 */
void magma_cfb_encrypt_blocks(struct MagmaCfbEnc *mode,
                              const MagmaBlock *blocks_in,
                              MagmaBlock *blocks_out,
                              uintptr_t blocks_len);

/**
 * @brief Зашифровать сообщение.
 *
 * Зашифровывает сообщение с помощью заданного контекста #MagmaCfbEnc.
 *
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * \see #MagmaCfbEnc
 */
void magma_cfb_encrypt(struct MagmaCfbEnc *mode,
                       const uint8_t *buf_in,
                       uint8_t *buf_out,
                       uintptr_t buf_len);

/**
 * @brief Дополнить и зашифровать сообщение.
 *
 * Зашифровывает сообщение с помощью заданного контекста #MagmaCfbEnc. Уничтожает
 * контекст шифра.
 *
 * Байты `0..pos` буфера `buf` используются как сообщение, когда же байты `pos..buf_len`
 * используются как зарезервированное место для дополнения.
 *
 * Возвращает ошибку в случае, когда место для дополнения недостаточно велико.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_in_len — длина буфера `buf_in` (в байтах).
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_out_len — длина буфера `buf_out` (в байтах).
 *
 * @returns Длину шифротекста, в случае успеха или -1, в случае ошибки.
 *
 * \see #MagmaCfbEnc
 */
intptr_t magma_cfb_encrypt_pad(struct MagmaCfbEnc *mode,
                               const uint8_t *buf_in,
                               uintptr_t buf_in_len,
                               uint8_t *buf_out,
                               uintptr_t buf_out_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_cfb_enc_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #MagmaCfbEnc
 */
void magma_cfb_enc_drop(struct MagmaCfbEnc *mode);

/**
 * @brief Инициализировать состояние блочного режима на основе ссылки на блочный шифр.
 *
 * Создаёт контекст блочного режима с использованием указателя на блочный шифр `cipher`
 * и синхропосылки `nonce`. Память с контекстом блочного шифра должна быть действительна
 * на всём протяжении жизни созданного контекста блочного режима. Иначе говоря, созданный
 * контекст хранит внутри себя указатель `cipher`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param cipher — ненулевой указатель на инициализированное состояние блочного шифра #Kuznyechik.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #KuznyechikRefCfbDec
 */
void kuznyechik_ref_cfb_dec_new(struct KuznyechikRefCfbDec *dst,
                                const struct KuznyechikEnc *cipher,
                                const KuznyechikNonce *nonce);

/**
 * @brief Расшифровать массив блоков.
 *
 * Расшифровывает блоки данных с помощью заданного контекста #KuznyechikRefCfbDec.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks_in — ненулевой указатель на буфер содержащий входные блоки.
 * @param blocks_out — ненулевой указатель на буфер для записи полученных блоков.
 * @param blocks_len — длина буферов blocks_in и blocks_out (в блоках).
 *
 * \see #KuznyechikRefCfbDec
 */
void kuznyechik_ref_cfb_decrypt_blocks(struct KuznyechikRefCfbDec *mode,
                                       const KuznyechikBlock *blocks_in,
                                       KuznyechikBlock *blocks_out,
                                       uintptr_t blocks_len);

/**
 * @brief Расшифровать сообщение.
 *
 * Зашифровывает сообщение с помощью заданного контекста #KuznyechikRefCfbDec.
 *
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * \see #KuznyechikRefCfbDec
 */
void kuznyechik_ref_cfb_decrypt(struct KuznyechikRefCfbDec *mode,
                                const uint8_t *buf_in,
                                uint8_t *buf_out,
                                uintptr_t buf_len);

/**
 * @brief Расшифровать сообщение и убрать дополнение.
 *
 * Расшифровывает сообщение с помощью заданного контекста #KuznyechikRefCfbDec. Уничтожает
 * контекст шифра.
 *
 * Возвращает ошибку в случае, когда длина буфера не делится на #MAGMA_BLOCK_SIZE или
 * расшифрованное сообщение имеет некорректное дополнение.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * @returns Длину результата, в случае успеха или -1, в случае ошибки.
 *
 * \see #KuznyechikRefCfbDec
 */
intptr_t kuznyechik_ref_cfb_decrypt_unpad(struct KuznyechikRefCfbDec *mode,
                                          const uint8_t *buf_in,
                                          uint8_t *buf_out,
                                          uintptr_t buf_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_ref_cfb_dec_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #KuznyechikRefCfbDec
 */
void kuznyechik_ref_cfb_dec_drop(struct KuznyechikRefCfbDec *mode);

/**
 * @brief Инициализировать состояние блочного режима на основе ссылки на блочный шифр.
 *
 * Создаёт контекст блочного режима с использованием указателя на блочный шифр `cipher`
 * и синхропосылки `nonce`. Память с контекстом блочного шифра должна быть действительна
 * на всём протяжении жизни созданного контекста блочного режима. Иначе говоря, созданный
 * контекст хранит внутри себя указатель `cipher`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param cipher — ненулевой указатель на инициализированное состояние блочного шифра #Kuznyechik.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #KuznyechikRefCfbEnc
 */
void kuznyechik_ref_cfb_enc_new(struct KuznyechikRefCfbEnc *dst,
                                const struct KuznyechikEnc *cipher,
                                const KuznyechikNonce *nonce);

/**
 * @brief Зашифровать массив блоков.
 *
 * Шифрует блоки данных с помощью заданного контекста #KuznyechikRefCfbEnc.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks_in — ненулевой указатель на буфер входных блоков.
 * @param blocks_out — ненулевой указатель для записи полученных блоков.
 * @param blocks_len — длина буферов blocks_in и blocks_out (в блоках).
 *
 * \see #KuznyechikRefCfbEnc
 */
void kuznyechik_ref_cfb_encrypt_blocks(struct KuznyechikRefCfbEnc *mode,
                                       const KuznyechikBlock *blocks_in,
                                       KuznyechikBlock *blocks_out,
                                       uintptr_t blocks_len);

/**
 * @brief Зашифровать сообщение.
 *
 * Зашифровывает сообщение с помощью заданного контекста #KuznyechikRefCfbEnc.
 *
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param in_buf — ненулевой указатель на буфер содержащий входное сообщение.
 * @param out_buf — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * \see #KuznyechikRefCfbEnc
 */
void kuznyechik_ref_cfb_encrypt(struct KuznyechikRefCfbEnc *mode,
                                const uint8_t *in_buf,
                                uint8_t *out_buf,
                                uintptr_t buf_len);

/**
 * @brief Дополнить и зашифровать сообщение.
 *
 * Зашифровывает сообщение с помощью заданного контекста #KuznyechikRefCfbEnc. Уничтожает
 * контекст шифра.
 *
 * Байты `0..pos` буфера `buf` используются как сообщение, когда же байты `pos..buf_len`
 * используются как зарезервированное место для дополнения.
 *
 * Возвращает ошибку в случае, когда место для дополнения недостаточно велико.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_in_len — длина буфера `buf_in` (в байтах).
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_out_len — длина буфера `buf_out` (в байтах).
 *
 * @returns Длину шифротекста, в случае успеха или -1, в случае ошибки.
 *
 * \see #KuznyechikRefCfbEnc
 */
intptr_t kuznyechik_ref_cfb_encrypt_pad(struct KuznyechikRefCfbEnc *mode,
                                        const uint8_t *buf_in,
                                        uintptr_t buf_in_len,
                                        uint8_t *buf_out,
                                        uintptr_t buf_out_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_ref_cfb_enc_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #KuznyechikRefCfbEnc
 */
void kuznyechik_ref_cfb_enc_drop(struct KuznyechikRefCfbEnc *mode);

/**
 * @brief Инициализировать состояние блочного режима на основе ссылки на блочный шифр.
 *
 * Создаёт контекст блочного режима с использованием указателя на блочный шифр `cipher`
 * и синхропосылки `nonce`. Память с контекстом блочного шифра должна быть действительна
 * на всём протяжении жизни созданного контекста блочного режима. Иначе говоря, созданный
 * контекст хранит внутри себя указатель `cipher`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param cipher — ненулевой указатель на инициализированное состояние блочного шифра #Magma.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #MagmaRefCfbDec
 */
void magma_ref_cfb_dec_new(struct MagmaRefCfbDec *dst,
                           const struct Magma *cipher,
                           const MagmaNonce *nonce);

/**
 * @brief Расшифровать массив блоков.
 *
 * Расшифровывает блоки данных с помощью заданного контекста #MagmaRefCfbDec.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks_in — ненулевой указатель на буфер содержащий входные блоки.
 * @param blocks_out — ненулевой указатель на буфер для записи полученных блоков.
 * @param blocks_len — длина буферов blocks_in и blocks_out (в блоках).
 *
 * \see #MagmaRefCfbDec
 */
void magma_ref_cfb_decrypt_blocks(struct MagmaRefCfbDec *mode,
                                  const MagmaBlock *blocks_in,
                                  MagmaBlock *blocks_out,
                                  uintptr_t blocks_len);

/**
 * @brief Расшифровать сообщение.
 *
 * Зашифровывает сообщение с помощью заданного контекста #MagmaRefCfbDec.
 *
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * \see #MagmaRefCfbDec
 */
void magma_ref_cfb_decrypt(struct MagmaRefCfbDec *mode,
                           const uint8_t *buf_in,
                           uint8_t *buf_out,
                           uintptr_t buf_len);

/**
 * @brief Расшифровать сообщение и убрать дополнение.
 *
 * Расшифровывает сообщение с помощью заданного контекста #MagmaRefCfbDec. Уничтожает
 * контекст шифра.
 *
 * Возвращает ошибку в случае, когда длина буфера не делится на #MAGMA_BLOCK_SIZE или
 * расшифрованное сообщение имеет некорректное дополнение.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * @returns Длину результата, в случае успеха или -1, в случае ошибки.
 *
 * \see #MagmaRefCfbDec
 */
intptr_t magma_ref_cfb_decrypt_unpad(struct MagmaRefCfbDec *mode,
                                     const uint8_t *buf_in,
                                     uint8_t *buf_out,
                                     uintptr_t buf_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_ref_cfb_dec_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #MagmaRefCfbDec
 */
void magma_ref_cfb_dec_drop(struct MagmaRefCfbDec *mode);

/**
 * @brief Инициализировать состояние блочного режима на основе ссылки на блочный шифр.
 *
 * Создаёт контекст блочного режима с использованием указателя на блочный шифр `cipher`
 * и синхропосылки `nonce`. Память с контекстом блочного шифра должна быть действительна
 * на всём протяжении жизни созданного контекста блочного режима. Иначе говоря, созданный
 * контекст хранит внутри себя указатель `cipher`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения состояния блочного режима.
 * @param cipher — ненулевой указатель на инициализированное состояние блочного шифра #Magma.
 * @param nonce — ненулевой указатель на синхропосылку.
 *
 * \see #MagmaRefCfbEnc
 */
void magma_ref_cfb_enc_new(struct MagmaRefCfbEnc *dst,
                           const struct Magma *cipher,
                           const MagmaNonce *nonce);

/**
 * @brief Зашифровать массив блоков.
 *
 * Шифрует блоки данных с помощью заданного контекста #MagmaRefCfbEnc.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param blocks_in — ненулевой указатель на буфер содержащий входные блоки.
 * @param blocks_out — ненулевой указатель на буфер для записи полученных блоков.
 * @param blocks_len — длина буферов blocks_in и blocks_out (в блоках).
 *
 * \see #MagmaRefCfbEnc
 */
void magma_ref_cfb_encrypt_blocks(struct MagmaRefCfbEnc *mode,
                                  const MagmaBlock *blocks_in,
                                  MagmaBlock *blocks_out,
                                  uintptr_t blocks_len);

/**
 * @brief Зашифровать сообщение.
 *
 * Зашифровывает сообщение с помощью заданного контекста #MagmaRefCfbEnc.
 *
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * \see #MagmaRefCfbEnc
 */
void magma_ref_cfb_encrypt(struct MagmaRefCfbEnc *mode,
                           const uint8_t *buf_in,
                           uint8_t *buf_out,
                           uintptr_t buf_len);

/**
 * @brief Дополнить и зашифровать сообщение.
 *
 * Зашифровывает сообщение с помощью заданного контекста #MagmaRefCfbEnc. Уничтожает
 * контекст шифра.
 *
 * Байты `0..pos` буфера `buf` используются как сообщение, когда же байты `pos..buf_len`
 * используются как зарезервированное место для дополнения.
 *
 * Возвращает ошибку в случае, когда место для дополнения недостаточно велико.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_in_len — длина буфера `buf_in` (в байтах).
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_out_len — длина буфера `buf_out` (в байтах).
 *
 * @returns Длину шифротекста, в случае успеха или -1, в случае ошибки.
 *
 * \see #MagmaRefCfbEnc
 */
intptr_t magma_ref_cfb_encrypt_pad(struct MagmaRefCfbEnc *mode,
                                   const uint8_t *buf_in,
                                   uintptr_t buf_in_len,
                                   uint8_t *buf_out,
                                   uintptr_t buf_out_len);

/**
 * @brief Уничтожить состояние блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_ref_cfb_enc_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на состояние блочного режима.
 *
 * \see #MagmaRefCfbEnc
 */
void magma_ref_cfb_enc_drop(struct MagmaRefCfbEnc *mode);

/**
 * @brief Создать контекст блочного режима.
 *
 * Создаёт контекст блочного режима в режиме гаммирования с использованием ключа `key`
 * и синхропосылки `nonce`
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения контекста блочного режима.
 * @param key — ненулевой указатель на ключ.
 * @param nonce — ненулевой указатель синхропосылку.
 *
 * \see #KuznyechikCtr
 */
void kuznyechik_ctr_new(struct KuznyechikCtr *dst,
                        const KuznyechikKey *key,
                        const KuznyechikCtrNonce *nonce);

/**
 * @brief Гаммировать сообщение c гамма-последовательностью блочного режима.
 *
 * Шифрует или расшифровывает сообщение посредством гаммирования его с гамма-последовательностью
 * шифра. Возвращает ошибку в случае, когда достигнут конец гаммы-последовательности.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * @returns 0, в случае успеха, -1, в случае ошибки.
 *
 * \see #KuznyechikCtr
 */
intptr_t kuznyechik_ctr_apply_keystream(struct KuznyechikCtr *mode,
                                        const uint8_t *buf_in,
                                        uint8_t *buf_out,
                                        uintptr_t buf_len);

/**
 * @brief Установить позицию гамма-последовательности блочного режима в байтах.
 *
 * Устанавливает текущую позицию гамма-последовательности блочного режима. Возвращает ошибку
 * в случае, когда требуемая позиция превышает длину гаммы-последовательности.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param pos — новая позиция гамма-последовательности (в байтах).
 *
 * @returns 0, в случае успеха, -1, в случае ошибки.
 *
 * \see #KuznyechikCtr
 */
intptr_t kuznyechik_ctr_seek(struct KuznyechikCtr *mode,
                             uint64_t pos);

/**
 * @brief Получить текущую позицию гамма-последовательности блочного режима в байтах.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 *
 * @returns Текущая позиция гамма-последовательности (в байтах), либо -1 в случае ошибки.
 *
 * \see #KuznyechikCtr
 */
intptr_t kuznyechik_ctr_get_pos(const struct KuznyechikCtr *mode,
                                uint64_t *pos);

/**
 * @brief Уничтожить контекст блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_ref_ctr_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 *
 * \see #KuznyechikCtr
 */
void kuznyechik_ctr_drop(struct KuznyechikCtr *mode);

/**
 * @brief Создать контекст блочного режима.
 *
 * Создаёт контекст блочного режима в режиме гаммирования с использованием ключа `key`
 * и синхропосылки `nonce`
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения контекста блочного режима.
 * @param key — ненулевой указатель на ключ.
 * @param nonce — ненулевой указатель синхропосылку.
 *
 * \see #MagmaCtr
 */
void magma_ctr_new(struct MagmaCtr *dst,
                   const MagmaKey *key,
                   const MagmaCtrNonce *nonce);

/**
 * @brief Гаммировать сообщение c гамма-последовательностью блочного режима.
 *
 * Шифрует или расшифровывает сообщение посредством гаммирования его с гамма-последовательностью
 * шифра. Возвращает ошибку в случае, когда достигнут конец гаммы-последовательности.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * @returns 0, в случае успеха, -1, в случае ошибки.
 *
 * \see #MagmaCtr
 */
intptr_t magma_ctr_apply_keystream(struct MagmaCtr *mode,
                                   const uint8_t *buf_in,
                                   uint8_t *buf_out,
                                   uintptr_t buf_len);

/**
 * @brief Установить позицию гамма-последовательности блочного режима в байтах.
 *
 * Устанавливает текущую позицию гамма-последовательности блочного режима. Возвращает ошибку
 * в случае, когда требуемая позиция превышает длину гаммы-последовательности.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param pos — новая позиция гамма-последовательности (в байтах).
 *
 * @returns 0, в случае успеха, -1, в случае ошибки.
 *
 * \see #MagmaCtr
 */
intptr_t magma_ctr_seek(struct MagmaCtr *mode,
                        uint64_t pos);

/**
 * @brief Получить текущую позицию гамма-последовательности блочного режима в байтах.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 *
 * @returns Текущая позиция гамма-последовательности (в байтах), либо -1 в случае ошибки.
 *
 * \see #MagmaCtr
 */
intptr_t magma_ctr_get_pos(const struct MagmaCtr *mode,
                           uint64_t *pos);

/**
 * @brief Уничтожить контекст блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_ref_ctr_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 *
 * \see #MagmaCtr
 */
void magma_ctr_drop(struct MagmaCtr *mode);

/**
 * @brief Создать контекст блочного режима на основе ссылки на блочный шифр.
 *
 * Создаёт контекст блочного режима с использованием указателя на блочный шифр `cipher`
 * и синхропосылки `nonce`. Память с контекстом блочного шифра должна быть действительна
 * на всём протяжении жизни созданного контекста блочного режима. Иначе говоря, созданный
 * контекст хранит внутри себя указатель `cipher`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения контекста блочного режима.
 * @param cipher — ненулевой указатель на инициализированное состояние блочного шифра #KuznyechikEnc.
 * @param nonce — ненулевой указатель синхропосылку.
 *
 * \see #KuznyechikRefCtr
 */
void kuznyechik_ref_ctr_new(struct KuznyechikRefCtr *dst,
                            const struct KuznyechikEnc *cipher,
                            const KuznyechikCtrNonce *nonce);

/**
 * @brief Гаммировать сообщение c гамма-последовательностью блочного режима.
 *
 * Шифрует или расшифровывает сообщение посредством гаммирования его с гамма-последовательностью
 * шифра. Возвращает ошибку в случае, когда достигнут конец гаммы-последовательности.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * @returns 0, в случае успеха, -1, в случае ошибки.
 *
 * \see #KuznyechikRefCtr
 */
intptr_t kuznyechik_ref_ctr_apply_keystream(struct KuznyechikRefCtr *mode,
                                            const uint8_t *buf_in,
                                            uint8_t *buf_out,
                                            uintptr_t buf_len);

/**
 * @brief Установить позицию гамма-последовательности блочного режима в байтах.
 *
 * Устанавливает текущую позицию гамма-последовательности блочного режима. Возвращает ошибку
 * в случае, когда требуемая позиция превышает длину гаммы-последовательности.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param pos — новая позиция гамма-последовательности (в байтах).
 *
 * @returns 0, в случае успеха, -1, в случае ошибки.
 *
 * \see #KuznyechikRefCtr
 */
intptr_t kuznyechik_ref_ctr_seek(struct KuznyechikRefCtr *mode,
                                 uint64_t pos);

/**
 * @brief Получить текущую позицию гамма-последовательности блочного режима в байтах.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 *
 * @returns Текущая позиция гамма-последовательности (в байтах), либо -1 в случае ошибки.
 *
 * \see #KuznyechikRefCtr
 */
intptr_t kuznyechik_ref_ctr_get_pos(const struct KuznyechikRefCtr *mode,
                                    uint64_t *pos);

/**
 * @brief Уничтожить контекст блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_ref_ctr_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 *
 * \see #KuznyechikRefCtr
 */
void kuznyechik_ref_ctr_drop(struct KuznyechikRefCtr *mode);

/**
 * @brief Создать контекст блочного режима на основе ссылки на блочный шифр.
 *
 * Создаёт контекст блочного режима с использованием указателя на блочный шифр `cipher`
 * и синхропосылки `nonce`. Память с контекстом блочного шифра должна быть действительна
 * на всём протяжении жизни созданного контекста блочного режима. Иначе говоря, созданный
 * контекст хранит внутри себя указатель `cipher`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения контекста блочного режима.
 * @param cipher — ненулевой указатель на инициализированное состояние блочного шифра #Magma.
 * @param nonce — ненулевой указатель синхропосылку.
 *
 * \see #MagmaRefCtr
 */
void magma_ref_ctr_new(struct MagmaRefCtr *dst,
                       const struct Magma *cipher,
                       const MagmaCtrNonce *nonce);

/**
 * @brief Гаммировать сообщение c гамма-последовательностью блочного режима.
 *
 * Шифрует или расшифровывает сообщение посредством гаммирования его с гамма-последовательностью
 * шифра. Возвращает ошибку в случае, когда достигнут конец гаммы-последовательности.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * @returns 0, в случае успеха, -1, в случае ошибки.
 *
 * \see #MagmaRefCtr
 */
intptr_t magma_ref_ctr_apply_keystream(struct MagmaRefCtr *mode,
                                       const uint8_t *buf_in,
                                       uint8_t *buf_out,
                                       uintptr_t buf_len);

/**
 * @brief Установить позицию гамма-последовательности блочного режима в байтах.
 *
 * Устанавливает текущую позицию гамма-последовательности блочного режима. Возвращает ошибку
 * в случае, когда требуемая позиция превышает длину гаммы-последовательности.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param pos — новая позиция гамма-последовательности (в байтах).
 *
 * @returns 0, в случае успеха, -1, в случае ошибки.
 *
 * \see #MagmaRefCtr
 */
intptr_t magma_ref_ctr_seek(struct MagmaRefCtr *mode,
                            uint64_t pos);

/**
 * @brief Получить текущую позицию гамма-последовательности блочного режима в байтах.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 *
 * @returns Текущая позиция гамма-последовательности (в байтах), либо -1 в случае ошибки.
 *
 * \see #MagmaRefCtr
 */
intptr_t magma_ref_ctr_get_pos(const struct MagmaRefCtr *mode,
                               uint64_t *pos);

/**
 * @brief Уничтожить контекст блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_ref_ctr_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 *
 * \see #MagmaRefCtr
 */
void magma_ref_ctr_drop(struct MagmaRefCtr *mode);

/**
 * @brief Создать контекст блочного режима.
 *
 * Создаёт контекст блочного режима в режиме гаммирования с обратной связью по выходу
 * с использованием ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения контекста блочного режима.
 * @param key — ненулевой указатель на ключ.
 * @param nonce — ненулевой указатель синхропосылку.
 *
 * \see #KuznyechikOfb
 */
void kuznyechik_ofb_new(struct KuznyechikOfb *dst,
                        const KuznyechikKey *key,
                        const KuznyechikNonce *nonce);

/**
 * @brief Гаммировать сообщение c гамма-последовательностью блочного режима.
 *
 * Шифрует или расшифровывает сообщение посредством гаммирования
 * c гамма-последовательностью блочного режима.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * \see #KuznyechikOfb
 */
void kuznyechik_ofb_apply_keystream(struct KuznyechikOfb *mode,
                                    const uint8_t *buf_in,
                                    uint8_t *buf_out,
                                    uintptr_t buf_len);

/**
 * @brief Уничтожить контекст блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_ofb_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 *
 * \see #KuznyechikOfb
 */
void kuznyechik_ofb_drop(struct KuznyechikOfb *mode);

/**
 * @brief Создать контекст блочного режима.
 *
 * Создаёт контекст блочного режима в режиме гаммирования с обратной связью по выходу
 * с использованием ключа `key` и синхропосылки `nonce`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения контекста блочного режима.
 * @param key — ненулевой указатель на ключ.
 * @param nonce — ненулевой указатель синхропосылку.
 *
 * \see #MagmaOfb
 */
void magma_ofb_new(struct MagmaOfb *dst,
                   const MagmaKey *key,
                   const MagmaNonce *nonce);

/**
 * @brief Гаммировать сообщение c гамма-последовательностью блочного режима.
 *
 * Шифрует или расшифровывает сообщение посредством гаммирования
 * c гамма-последовательностью блочного режима.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * \see #MagmaOfb
 */
void magma_ofb_apply_keystream(struct MagmaOfb *mode,
                               const uint8_t *buf_in,
                               uint8_t *buf_out,
                               uintptr_t buf_len);

/**
 * @brief Уничтожить контекст блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_ofb_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 *
 * \see #MagmaOfb
 */
void magma_ofb_drop(struct MagmaOfb *mode);

/**
 * @brief Создать контекст блочного режима на основе ссылки на блочный шифр.
 *
 * Создаёт контекст блочного режима с использованием указателя на блочный шифр `cipher`
 * и синхропосылки `nonce`. Память с контекстом блочного шифра должна быть действительна
 * на всём протяжении жизни созданного контекста блочного режима. Иначе говоря, созданный
 * контекст хранит внутри себя указатель `cipher`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения контекста блочного режима.
 * @param cipher — ненулевой указатель на инициализированное состояние блочного шифра #Kuznyechik.
 * @param nonce — ненулевой указатель синхропосылку.
 *
 * \see #KuznyechikRefOfb
 */
void kuznyechik_ref_ofb_new(struct KuznyechikRefOfb *dst,
                            const struct KuznyechikEnc *cipher,
                            const KuznyechikNonce *nonce);

/**
 * @brief Гаммировать сообщение c гамма-последовательностью блочного режима.
 *
 * Шифрует или расшифровывает сообщение посредством гаммирования
 * c гамма-последовательностью блочного режима.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * \see #KuznyechikRefOfb
 */
void kuznyechik_ref_ofb_apply_keystream(struct KuznyechikRefOfb *mode,
                                        const uint8_t *buf_in,
                                        uint8_t *buf_out,
                                        uintptr_t buf_len);

/**
 * @brief Уничтожить контекст блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #kuznyechik_ref_ofb_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 *
 * \see #KuznyechikRefOfb
 */
void kuznyechik_ref_ofb_drop(struct KuznyechikRefOfb *mode);

/**
 * @brief Создать контекст блочного режима на основе ссылки на блочный шифр.
 *
 * Создаёт контекст блочного режима с использованием указателя на блочный шифр `cipher`
 * и синхропосылки `nonce`. Память с контекстом блочного шифра должна быть действительна
 * на всём протяжении жизни созданного контекста блочного режима. Иначе говоря, созданный
 * контекст хранит внутри себя указатель `cipher`.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param dst — ненулевой указатель на область памяти для хранения контекста блочного режима.
 * @param cipher — ненулевой указатель на инициализированное состояние блочного шифра #Magma.
 * @param nonce — ненулевой указатель синхропосылку.
 *
 * \see #MagmaRefOfb
 */
void magma_ref_ofb_new(struct MagmaRefOfb *dst,
                       const struct Magma *cipher,
                       const MagmaNonce *nonce);

/**
 * @brief Гаммировать сообщение c гамма-последовательностью блочного режима.
 *
 * Шифрует или расшифровывает сообщение посредством гаммирования
 * c гамма-последовательностью блочного режима.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 * @param buf_in — ненулевой указатель на буфер содержащий входное сообщение.
 * @param buf_out — ненулевой указатель на буфер для записи результата.
 * @param buf_len — длина буферов (в байтах).
 *
 * \see #MagmaRefOfb
 */
void magma_ref_ofb_apply_keystream(struct MagmaRefOfb *mode,
                                   const uint8_t *buf_in,
                                   uint8_t *buf_out,
                                   uintptr_t buf_len);

/**
 * @brief Уничтожить контекст блочного режима.
 *
 * Очищает переданный контекст и обнуляет память. Контекст должен быть предварительно
 * создан с помощью функции #magma_ref_ofb_new или скопирован из существующего.
 *
 * \warning Передача нулевого или некорректного указателя приводит к неопределённому поведению.
 * \warning Использование указателя на контекст после вызова данной функции приводит
 * к неопределённому поведению.
 *
 * @param mode — ненулевой указатель на контекст блочного режима.
 *
 * \see #MagmaRefOfb
 */
void magma_ref_ofb_drop(struct MagmaRefOfb *mode);

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus

#endif /* GOSTCRYPTO_H */
